import os
import platform
import textwrap
import unittest
from textwrap import dedent

import pytest

from parameterized.parameterized import parameterized

from conans.client.build.cmake import CMake
from conans.model.version import Version
from conans.test.utils.tools import TestClient

conanfile_py = """
from conans import ConanFile

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    build_policy="missing"
    def package_info(self):
        self.cpp_info.cxxflags = ["MyFlag1", "MyFlag2"]
        self.cpp_info.cflags = ["-load", "C:\some\path"]
        self.cpp_info.defines = ['MY_DEF=My" \string', 'MY_DEF2=My${} other \string']
"""

chatconanfile_py = """
from conans import ConanFile

class ChatConan(ConanFile):
    name = "Chat"
    version = "0.1"
    requires = "Hello/0.1@lasote/testing"
    build_policy="missing"
    def package_info(self):
        self.cpp_info.cxxflags = ["MyChatFlag1", "MyChatFlag2"]
"""

conanfile = """[requires]
Hello/0.1@lasote/testing
"""

cmake = """cmake_minimum_required(VERSION 2.8.12)
set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
project(MyHello CXX)

include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
conan_basic_setup()

message(STATUS "CMAKE_CXX_FLAGS=${CMAKE_CXX_FLAGS}")
message(STATUS "CONAN_CXX_FLAGS=${CONAN_CXX_FLAGS}")
message(STATUS "CMAKE_C_FLAGS=${CMAKE_C_FLAGS}")
message(STATUS "CONAN_C_FLAGS=${CONAN_C_FLAGS}")
message(STATUS "HELLO_CXX_FLAGS=${HELLO_FLAGS}")
message(STATUS "CHAT_CXX_FLAGS=${CHAT_FLAGS}")
message(STATUS "CONAN_DEFINES_HELLO=${CONAN_DEFINES_HELLO}")
message(STATUS "HELLO_DEFINES=${HELLO_DEFINES}")
"""


@pytest.mark.slow
@pytest.mark.tool_cmake
class CMakeFlagsTest(unittest.TestCase):

    def _get_line(self, text, begin):
        lines = str(text).splitlines()
        begin = "-- %s=" % begin
        line = [l for l in lines if l.startswith(begin)][0]
        flags = line[len(begin):].strip()
        self.assertNotIn("'", flags)
        self.assertNotIn('"', flags)
        return flags

    @pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows for vcvars")
    def test_vcvars_priority(self):
        # https://github.com/conan-io/conan/issues/5999
        client = TestClient()
        conanfile_vcvars = textwrap.dedent("""
            import os
            from conans import ConanFile, CMake

            class HelloConan(ConanFile):
                settings = "os", "compiler", "arch", "build_type"
                def build(self):
                    cmake = CMake(self, generator="Ninja", append_vcvars=True)
                    cmake.configure()

                # CAPTURING THE RUN METHOD
                def run(self, cmd):
                    self.output.info("PATH ENV VAR: %s" % os.getenv("PATH"))
            """)

        client.save({"conanfile.py": conanfile_vcvars})
        # FIXME this would fail:
        # client.run('create . pkg/1.0@ -e PATH="MyCustomPath"')
        # because cmake will not be in the PATH anymore, and CMake.get_version() fails
        # For some reason cmake.configure() worked in the past, because it is finding the
        # cmake inside VISUAL STUDIO!!! (cmake version 3.12.18081601-MSVC_2), because VS vcvars
        # is activated by CMake for Ninja
        client.run('create . pkg/1.0@ -e PATH=["MyCustomPath"]')
        self.assertIn("pkg/1.0: PATH ENV VAR: MyCustomPath;", client.out)

    @parameterized.expand([(True, ), (False, )])
    def test_build_app(self, targets):
        client = TestClient()
        conanfile_py = """
from conans import ConanFile

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    def package_info(self):
        self.cpp_info.defines = [r'MY_DEF=My${} $string', r'MY_DEF2=My$ other string']
"""
        client.save({"conanfile.py": conanfile_py})
        client.run("create . lasote/testing")
        consumer = """from conans import ConanFile, CMake
import os
class App(ConanFile):
    settings = "os", "compiler", "arch", "build_type"
    requires = "Hello/0.1@lasote/testing"
    generators = "cmake"
    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()
        self.run(os.sep.join([".", "bin", "myapp"]))
"""
        cmake_app = """cmake_minimum_required(VERSION 2.8.12)
set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
project(MyHello CXX)

include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
conan_basic_setup(%s)
add_executable(myapp myapp.cpp)
conan_target_link_libraries(myapp)
""" % ("TARGETS" if targets else "")

        myapp = r"""#include <iostream>
#define STRINGIFY(x) #x
#define STRINGIFYMACRO(y) STRINGIFY(y)
int main(){
    std::cout << "Msg1: " << STRINGIFYMACRO(MY_DEF) << "\n";
    std::cout << "Msg2: " << STRINGIFYMACRO(MY_DEF2) << "\n";
}"""
        client.save({"conanfile.py": consumer,
                     "CMakeLists.txt": cmake_app,
                     "myapp.cpp": myapp
                     }, clean_first=True)
        client.run("install .")
        client.run("build .")
        self.assertIn("Msg1: My${} $string", client.out)
        self.assertIn("Msg2: My$ other string", client.out)

    def test_flags(self):
        client = TestClient()
        client.save({"conanfile.py": conanfile_py})
        client.run("export . lasote/testing")
        client.save({"conanfile.txt": conanfile,
                     "CMakeLists.txt": cmake}, clean_first=True)

        client.run('install . -g cmake')
        generator = '-G "Visual Studio 15 Win64"' if platform.system() == "Windows" else ""
        client.run_command("cmake . %s" % generator)
        cmake_cxx_flags = self._get_line(client.out, "CMAKE_CXX_FLAGS")
        self.assertTrue(cmake_cxx_flags.endswith("MyFlag1 MyFlag2"))
        self.assertIn("CONAN_CXX_FLAGS=MyFlag1 MyFlag2", client.out)
        self.assertIn("CMAKE_C_FLAGS= -load C:\some\path", client.out)
        self.assertIn("CONAN_C_FLAGS=-load C:\some\path ", client.out)
        self.assertIn('CONAN_DEFINES_HELLO=-DMY_DEF=My" \string;-DMY_DEF2=My${} other \string',
                      client.out)

    def test_transitive_flags(self):
        client = TestClient()
        client.save({"conanfile.py": conanfile_py})
        client.run("export . lasote/testing")
        client.save({"conanfile.py": chatconanfile_py}, clean_first=True)
        client.run("export . lasote/testing")
        client.save({"conanfile.txt": conanfile.replace("Hello", "Chat"),
                     "CMakeLists.txt": cmake}, clean_first=True)

        client.run('install . -g cmake')
        generator = '-G "Visual Studio 15 Win64"' if platform.system() == "Windows" else ""
        client.run_command("cmake . %s" % generator)
        cmake_cxx_flags = self._get_line(client.out, "CMAKE_CXX_FLAGS")
        self.assertTrue(cmake_cxx_flags.endswith("MyFlag1 MyFlag2 MyChatFlag1 MyChatFlag2"))
        self.assertIn("CONAN_CXX_FLAGS=MyFlag1 MyFlag2 MyChatFlag1 MyChatFlag2",
                      client.out)

    def test_targets_flags(self):
        client = TestClient()
        client.save({"conanfile.py": conanfile_py})
        client.run("export . lasote/testing")
        cmake_targets = cmake.replace("conan_basic_setup()",
                                      "conan_basic_setup(TARGETS)\n"
                                      "get_target_property(HELLO_FLAGS CONAN_PKG::Hello"
                                      " INTERFACE_COMPILE_OPTIONS)\n"
                                      "get_target_property(HELLO_DEFINES CONAN_PKG::Hello"
                                      " INTERFACE_COMPILE_DEFINITIONS)")
        client.save({"conanfile.txt": conanfile,
                     "CMakeLists.txt": cmake_targets},
                    clean_first=True)

        client.run('install . -g cmake')
        generator = '-G "Visual Studio 15 Win64"' if platform.system() == "Windows" else ""
        client.run_command("cmake . %s" % generator)
        cmake_cxx_flags = self._get_line(client.out, "CMAKE_CXX_FLAGS")
        self.assertNotIn("My", cmake_cxx_flags)
        self.assertIn("CONAN_CXX_FLAGS=MyFlag1 MyFlag2", client.out)
        self.assertIn("HELLO_CXX_FLAGS=-load;C:\some\path;MyFlag1;MyFlag2;"
                      "$<$<CONFIG:Release>:;>;$<$<CONFIG:RelWithDebInfo>:;>;"
                      "$<$<CONFIG:MinSizeRel>:;>;$<$<CONFIG:Debug>:;>", client.out)
        self.assertIn('HELLO_DEFINES=MY_DEF=My" \string;MY_DEF2=My${} other \string;', client.out)

    def test_targets_own_flags(self):
        client = TestClient()
        client.save({"conanfile.py": conanfile_py.replace('version = "0.1"',
                                                          'version = "0.1"\n'
                                                          '    settings = "compiler"')})
        client.run("export . lasote/testing")
        cmake_targets = cmake.replace("conan_basic_setup()",
                                      "conan_basic_setup(TARGETS)\n"
                                      "get_target_property(HELLO_FLAGS CONAN_PKG::Hello"
                                      " INTERFACE_COMPILE_OPTIONS)\n"
                                      "get_target_property(HELLO_DEFINES CONAN_PKG::Hello"
                                      " INTERFACE_COMPILE_DEFINITIONS)")
        client.save({"conanfile.txt": conanfile,
                     "CMakeLists.txt": cmake_targets},
                    clean_first=True)

        client.run('install . -g cmake')
        generator = '-G "Visual Studio 15 Win64"' if platform.system() == "Windows" else ""
        client.run_command("cmake . %s -DCONAN_CXX_FLAGS=CmdCXXFlag" % generator)
        cmake_cxx_flags = self._get_line(client.out, "CMAKE_CXX_FLAGS")
        self.assertNotIn("My", cmake_cxx_flags)
        self.assertIn("CmdCXXFlag", cmake_cxx_flags)
        self.assertIn("CONAN_CXX_FLAGS=MyFlag1 MyFlag2 CmdCXXFlag", client.out)
        self.assertIn("HELLO_CXX_FLAGS=-load;C:\some\path;MyFlag1;MyFlag2;"
                      "$<$<CONFIG:Release>:;>;$<$<CONFIG:RelWithDebInfo>:;>;"
                      "$<$<CONFIG:MinSizeRel>:;>;$<$<CONFIG:Debug>:;>", client.out)
        self.assertIn('HELLO_DEFINES=MY_DEF=My" \string;MY_DEF2=My${} other \string;', client.out)

    def test_transitive_targets_flags(self):
        client = TestClient()
        client.save({"conanfile.py": conanfile_py})
        client.run("export . lasote/testing")
        client.save({"conanfile.py": chatconanfile_py}, clean_first=True)
        client.run("export . lasote/testing")
        cmake_targets = cmake.replace("conan_basic_setup()",
                                      "conan_basic_setup(TARGETS)\n"
                                      "get_target_property(HELLO_FLAGS CONAN_PKG::Hello"
                                      " INTERFACE_COMPILE_OPTIONS)\n"
                                      "get_target_property(CHAT_FLAGS CONAN_PKG::Chat"
                                      " INTERFACE_COMPILE_OPTIONS)\n"
                                      "get_target_property(HELLO_DEFINES CONAN_PKG::Hello"
                                      " INTERFACE_COMPILE_DEFINITIONS)")
        client.save({"conanfile.txt": conanfile.replace("Hello", "Chat"),
                     "CMakeLists.txt": cmake_targets},
                    clean_first=True)

        client.run('install . -g cmake')
        generator = '-G "Visual Studio 15 Win64"' if platform.system() == "Windows" else ""
        client.run_command("cmake . %s" % generator)

        cmake_cxx_flags = self._get_line(client.out, "CMAKE_CXX_FLAGS")
        self.assertNotIn("My", cmake_cxx_flags)
        self.assertIn("CONAN_CXX_FLAGS=MyFlag1 MyFlag2 MyChatFlag1 MyChatFlag2",
                      client.out)
        self.assertIn("HELLO_CXX_FLAGS=-load;C:\some\path;MyFlag1;MyFlag2;"
                      "$<$<CONFIG:Release>:;>;$<$<CONFIG:RelWithDebInfo>:;>;"
                      "$<$<CONFIG:MinSizeRel>:;>;$<$<CONFIG:Debug>:;>", client.out)
        self.assertIn("CHAT_CXX_FLAGS=MyChatFlag1;MyChatFlag2;"
                      "$<$<CONFIG:Release>:;>;$<$<CONFIG:RelWithDebInfo>:;>;"
                      "$<$<CONFIG:MinSizeRel>:;>;$<$<CONFIG:Debug>:;>", client.out)
        self.assertIn('HELLO_DEFINES=MY_DEF=My" \string;MY_DEF2=My${} other \string;', client.out)

    def test_cmake_needed_settings(self):
        conanfile = """
import os
from conans import ConanFile, CMake

class MyLib(ConanFile):
    name = "MyLib"
    version = "0.1"
    %s

    def build(self):
        cmake = CMake(self)
        """
        for settings_line in ('', 'settings="arch"', 'settings="compiler"'):
            client = TestClient()
            client.save({"conanfile.py": conanfile % settings_line})
            client.run("install .")
            client.run("build .")

    def test_cmake_shared_flag(self):
        conanfile = """
import os
from conans import ConanFile, CMake

class MyLib(ConanFile):
    name = "MyLib"
    version = "0.1"
    options = {"shared": [True, False]}
    default_options= "shared=%s"
    settings = "arch", "compiler"

    def build(self):
        cmake = CMake(self)
        if self.options.shared:
            assert(cmake.definitions["BUILD_SHARED_LIBS"] ==  "ON")
        else:
            assert(cmake.definitions["BUILD_SHARED_LIBS"] ==  "OFF")
        """
        client = TestClient()
        client.save({"conanfile.py": conanfile % "True"})
        client.run("build .", assert_error=True)

        self.assertIn("conanbuildinfo.txt file not found", client.out)

        client.run("install .")
        client.run("build .")

        client.save({"conanfile.py": conanfile % "False"}, clean_first=True)
        client.run("install .")
        client.run("build .")

    def test_std_flag_applied(self):
        conanfile = """
import os
from conans import ConanFile, CMake
class MyLib(ConanFile):
    name = "MyLib"
    version = "0.1"
    settings = "arch", "compiler", "cppstd"
    generators = "cmake"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()
        """
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "mylib.cpp": "auto myfunc(){return 3;}",  # c++14 feature
                     "CMakeLists.txt": """
cmake_minimum_required(VERSION 2.8.12)
set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
project(MyHello CXX)
include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
conan_basic_setup()
add_library(mylib mylib.cpp)
target_link_libraries(mylib ${CONAN_LIBS})
"""})

        if platform.system() != "Windows":
            client.run("install . --install-folder=build -s cppstd=gnu98")
            client.run("build . --build-folder=build", assert_error=True)
            self.assertIn("Error in build()", client.out)

            # Now specify c++14
            client.run("install . --install-folder=build -s cppstd=gnu14")
            client.run("build . --build-folder=build")
            self.assertIn("CPP STANDARD: 14 WITH EXTENSIONS ON", client.out)
            libname = "libmylib.a" if platform.system() != "Windows" else "mylib.lib"
            libpath = os.path.join(client.current_folder, "build", "lib", libname)
            self.assertTrue(os.path.exists(libpath))

        client.run("install . --install-folder=build -s cppstd=14")
        client.run("build . --build-folder=build")
        self.assertIn("CPP STANDARD: 14 WITH EXTENSIONS OFF", client.out)
        self.assertNotIn("Conan setting CXX_FLAGS flags", client.out)
        libname = "libmylib.a" if platform.system() != "Windows" else "mylib.lib"
        libpath = os.path.join(client.current_folder, "build", "lib", libname)
        self.assertTrue(os.path.exists(libpath))

    @pytest.mark.tool_mingw64
    def test_standard_20_as_cxx_flag(self):
        # CMake (1-Jun-2018) do not support the 20 flag in CMAKE_CXX_STANDARD var
        conanfile = """
import os
from conans import ConanFile, CMake
class MyLib(ConanFile):
    name = "MyLib"
    version = "0.1"
    settings = "arch", "compiler", "cppstd"
    exports_sources = "CMakeLists.txt"
    generators = "cmake"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
"""
        cmakelists = """
cmake_minimum_required(VERSION 2.8.12)
set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
project(MyHello CXX)
include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
conan_set_std()
"""
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "CMakeLists.txt": cmakelists})

        def conan_set_std_branch():
            # Replicate logic from cmake_common definition of 'macro(conan_set_std)'
            cmake_version = CMake.get_version()
            return cmake_version < Version("3.12")

        client.run("create . user/channel -s cppstd=gnu20 -s compiler=gcc "
                   "-s compiler.version=8 -s compiler.libcxx=libstdc++11")
        if conan_set_std_branch():
            self.assertIn("Conan setting CXX_FLAGS flags: -std=gnu++2a", client.out)
        else:
            self.assertIn("Conan setting CPP STANDARD: 20 WITH EXTENSIONS ON", client.out)

        client.run("create . user/channel -s cppstd=20 -s compiler=gcc -s compiler.version=8 "
                   "-s compiler.libcxx=libstdc++11")
        if conan_set_std_branch():
            self.assertIn("Conan setting CXX_FLAGS flags: -std=c++2a", client.out)
        else:
            self.assertIn("Conan setting CPP STANDARD: 20 WITH EXTENSIONS OFF", client.out)

    def test_fpic_applied(self):
        conanfile = """
import os
from conans import ConanFile, CMake
class MyLib(ConanFile):
    name = "MyLib"
    version = "0.1"
    settings = "arch", "compiler"
    options = {"fPIC": [True, False]}
    default_options = "fPIC=False"
    generators = "cmake"
    exports_sources = "CMakeLists.txt"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
"""
        cmakelists = """
cmake_minimum_required(VERSION 2.8.12)
set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
project(MyHello CXX)
include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
conan_basic_setup()
"""
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "CMakeLists.txt": cmakelists})

        client.run("create . user/channel -o MyLib:fPIC=True")
        self.assertIn("Conan: Adjusting fPIC flag (ON)", client.out)

        client.run("create . user/channel -o MyLib:fPIC=False")
        self.assertIn("Conan: Adjusting fPIC flag (OFF)", client.out)

        client.save({"conanfile.py": conanfile.replace("fPIC", "fpic")}, clean_first=False)
        client.run("create . user/channel -o MyLib:fpic=True")
        self.assertNotIn("Conan: Adjusting fPIC flag (ON)", client.out)

        # Skip fpic adjustements in basic setup
        tmp = cmakelists.replace("conan_basic_setup()", "conan_basic_setup(SKIP_FPIC)")
        client.save({"CMakeLists.txt": tmp, "conanfile.py": conanfile}, clean_first=True)
        client.run("create . user/channel -o MyLib:fPIC=True")
        self.assertNotIn("Conan: Adjusting fPIC flag", client.out)

    def test_header_only_generator(self):
        """ Test cmake.install() is possible although Generetaor could not be deduced from
        settings
        """
        conanfile = dedent("""
        from conans import ConanFile, CMake

        class TestConan(ConanFile):
            name = "kk"
            version = "1.0"
            exports = "*"

            def package(self):
                cmake = CMake(self)
                self.output.info("Configure command: %s" % cmake.command_line)
                cmake.configure()
                cmake.install()
        """)
        cmakelists = dedent("""
        cmake_minimum_required(VERSION 3.3)
        project(test)

        install(DIRECTORY "${CMAKE_CURRENT_SOURCE_DIR}/include"
            DESTINATION "${CMAKE_INSTALL_INCLUDEDIR}")
        """)
        client = TestClient()
        client.save({"conanfile.py": conanfile, "CMakeLists.txt": cmakelists, "include/file.h": ""})
        client.run("create . danimtb/testing")
        if platform.system() == "Windows":
            self.assertIn("WARN: CMake generator could not be deduced from settings", client.out)
            self.assertIn('Configure command: -DCONAN_IN_LOCAL_CACHE="ON" '
                          '-DCMAKE_INSTALL_PREFIX=', client.out)
        else:
            self.assertIn('Configure command: -G "Unix Makefiles" '
                          '-DCONAN_IN_LOCAL_CACHE="ON" -DCMAKE_INSTALL_PREFIX=', client.out)



import textwrap
import unittest

import pytest

from conans.test.utils.tools import TestClient


@pytest.mark.tool_cmake
class CMakeBuildHelper(unittest.TestCase):
    def test_get_version_no_toolchain(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
        from conans import ConanFile, CMake
        class Pkg(ConanFile):
            def build(self):
                self.output.info("CMAKE_VERSION: %s" % CMake.get_version())
                cmake = CMake(self)
                self.output.info("CMAKE_VERSION (obj): %s" % cmake.get_version())
        """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/0.1@user/testing")
        self.assertIn("CMAKE_VERSION: 3", client.out)
        self.assertIn("CMAKE_VERSION (obj): 3", client.out)

    def test_get_version_toolchain(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
        from conans import ConanFile, CMake
        class Pkg(ConanFile):

            def build(self):
                self.output.info("CMAKE_VERSION: %s" % CMake.get_version())
                cmake = CMake(self)
                self.output.info("CMAKE_VERSION (obj): %s" % cmake.get_version())
        """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/0.1@user/testing")
        self.assertIn("CMAKE_VERSION: 3", client.out)
        self.assertIn("CMAKE_VERSION (obj): 3", client.out)



# coding=utf-8

import platform
import unittest

from conans.client.generators.virtualrunenv import VirtualRunEnvGenerator
from conans.test.utils.mocks import ConanFileMock


class VirtualRunEnvGeneratorTest(unittest.TestCase):

    activate_sh = "activate_run.sh"
    activate_bat = "activate_run.bat"
    activate_ps1 = "activate_run.ps1"
    environment_sh_env = "environment_run.sh.env"
    environment_bat_env = "environment_run.bat.env"
    environment_ps1_env = "environment_run.ps1.env"

    @classmethod
    def setUpClass(cls):
        conanfile = ConanFileMock()
        conanfile.deps_cpp_info["hello"].bin_paths = ["bin1", "bin2"]
        conanfile.deps_cpp_info["hello"].lib_paths = ["lib1", "lib2"]

        cls.generator = VirtualRunEnvGenerator(conanfile)
        cls.generator.output_path = "not-used"
        cls.result = cls.generator.content

    def test_output(self):
        keys = ["deactivate_run.sh", "activate_run.sh", self.environment_sh_env,
                "activate_run.ps1", "deactivate_run.ps1", self.environment_ps1_env]
        if platform.system() == "Windows":
            keys += ["activate_run.bat", "deactivate_run.bat", self.environment_bat_env]

        self.assertListEqual(sorted(keys), sorted(self.result.keys()))

    def test_environment(self):
        self.assertEqual(self.generator.env["PATH"], ["bin1", "bin2"])
        self.assertEqual(self.generator.env["LD_LIBRARY_PATH"], ["lib1", "lib2"])
        self.assertEqual(self.generator.env["DYLD_LIBRARY_PATH"], ["lib1", "lib2"])

    def test_scripts(self):
        content = self.result[self.environment_sh_env]
        self.assertIn('DYLD_LIBRARY_PATH="lib1":"lib2"${DYLD_LIBRARY_PATH:+:$DYLD_LIBRARY_PATH}',
                      content)
        self.assertIn('LD_LIBRARY_PATH="lib1":"lib2"${LD_LIBRARY_PATH:+:$LD_LIBRARY_PATH}', content)
        self.assertIn('PATH="bin1":"bin2"${PATH:+:$PATH}', content)

        if platform.system() == "Windows":
            content = self.result[self.environment_bat_env]
            self.assertIn('DYLD_LIBRARY_PATH=lib1;lib2;%DYLD_LIBRARY_PATH%', content)
            self.assertIn('LD_LIBRARY_PATH=lib1;lib2;%LD_LIBRARY_PATH%', content)
            self.assertIn('PATH=bin1;bin2;%PATH%', content)

            content = self.result[self.environment_ps1_env]
            self.assertIn('DYLD_LIBRARY_PATH=lib1;lib2;$env:DYLD_LIBRARY_PATH', content)
            self.assertIn('LD_LIBRARY_PATH=lib1;lib2;$env:LD_LIBRARY_PATH', content)
            self.assertIn('PATH=bin1;bin2;$env:PATH', content)
        else:
            content = self.result[self.environment_ps1_env]
            self.assertIn('DYLD_LIBRARY_PATH=lib1:lib2:$env:DYLD_LIBRARY_PATH', content)
            self.assertIn('LD_LIBRARY_PATH=lib1:lib2:$env:LD_LIBRARY_PATH', content)
            self.assertIn('PATH=bin1:bin2:$env:PATH', content)



import os
import platform
import textwrap

import pytest

from conans.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
@pytest.mark.tool_cmake
def test_xcodedeps_components():
    """
    tcp/1.0 is a lib without components
    network/1.0 lib that requires tcp/1.0 and has three components:
     - core
     - client -> requires core component and tcp::tcp
     - server -> requires core component and tcp::tcp

    chat/1.0 -> lib that requires network::client component

    We create an application called ChatApp that uses XcodeDeps for chat/1.0 dependency
    And check that we are not requiring more than the needed components,
    nothing from network::server for example
    """
    client = TestClient(path_with_spaces=False)

    client.run("new tcp/1.0 -m=cmake_lib")
    client.run("create . -tf=None")

    header = textwrap.dedent("""
        #pragma once
        void {name}();
        """)

    source = textwrap.dedent("""
        #include <iostream>
        #include "{name}.h"
        {include}

        void {name}(){{
            {call}
            #ifdef NDEBUG
            std::cout << "{name}/1.0: Hello World Release!" << std::endl;
            #else
            std::cout << "{name}/1.0: Hello World Debug!" << std::endl;
            #endif
        }}
        """)

    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(myproject CXX)

        find_package(tcp REQUIRED CONFIG)

        add_library(core src/core.cpp include/core.h)
        add_library(client src/client.cpp include/client.h)
        add_library(server src/server.cpp include/server.h)

        target_include_directories(core PUBLIC include)
        target_include_directories(client PUBLIC include)
        target_include_directories(server PUBLIC include)

        set_target_properties(core PROPERTIES PUBLIC_HEADER "include/core.h")
        set_target_properties(client PROPERTIES PUBLIC_HEADER "include/client.h")
        set_target_properties(server PROPERTIES PUBLIC_HEADER "include/server.h")

        target_link_libraries(client core tcp::tcp)
        target_link_libraries(server core tcp::tcp)

        install(TARGETS core client server)
        """)

    conanfile_py = textwrap.dedent("""
        import os

        from conan import ConanFile
        from conan.tools.cmake import CMake, cmake_layout


        class LibConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            exports_sources = "CMakeLists.txt", "src/*", "include/*"
            {requires}
            generators = "CMakeToolchain", "CMakeDeps"

            def layout(self):
                cmake_layout(self)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()

            def package(self):
                cmake = CMake(self)
                cmake.install()
            {package_info}
        """)

    network_pi = """
    def package_info(self):
        self.cpp_info.components["core"].libs = ["core"]
        self.cpp_info.components["core"].includedirs.append("include")
        self.cpp_info.components["core"].libdirs.append("lib")

        self.cpp_info.components["client"].libs = ["client"]
        self.cpp_info.components["client"].includedirs.append("include")
        self.cpp_info.components["client"].libdirs.append("lib")
        self.cpp_info.components["client"].requires.extend(["core", "tcp::tcp"])

        self.cpp_info.components["server"].libs = ["server"]
        self.cpp_info.components["server"].includedirs.append("include")
        self.cpp_info.components["server"].libdirs.append("lib")
        self.cpp_info.components["server"].requires.extend(["core", "tcp::tcp"])
    """

    client.save({
        "include/core.h": header.format(name="core"),
        "include/server.h": header.format(name="server"),
        "include/client.h": header.format(name="client"),
        "src/core.cpp": source.format(name="core", include="", call=""),
        "src/server.cpp": source.format(name="server", include='#include "core.h"\n#include "tcp.h"',
                                        call="core(); tcp();"),
        "src/client.cpp": source.format(name="client", include='#include "core.h"\n#include "tcp.h"',
                                        call="core(); tcp();"),
        "conanfile.py": conanfile_py.format(requires='requires= "tcp/1.0"', package_info=network_pi),
        "CMakeLists.txt": cmakelists,
    }, clean_first=True)

    client.run("create . network/1.0@")

    chat_pi = """
    def package_info(self):
        self.cpp_info.libs = ["chat"]
        self.cpp_info.includedirs.append("include")
        self.cpp_info.libdirs.append("lib")
        self.cpp_info.requires.append("network::client")
    """

    cmakelists_chat = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(chat CXX)
        find_package(network REQUIRED CONFIG)
        add_library(chat src/chat.cpp include/chat.h)
        target_include_directories(chat PUBLIC include)
        set_target_properties(chat PROPERTIES PUBLIC_HEADER "include/chat.h")
        target_link_libraries(chat network::client)
        install(TARGETS chat)
        """)

    client.save({
        "include/chat.h": header.format(name="chat"),
        "src/chat.cpp": source.format(name="chat", include='#include "client.h"', call="client();"),
        "conanfile.py": conanfile_py.format(requires='requires= "network/1.0"',
                                            package_info=chat_pi),
        "CMakeLists.txt": cmakelists_chat,
    }, clean_first=True)

    client.run("create . chat/1.0@")

    xcode_project = textwrap.dedent("""
        name: ChatApp
        fileGroups:
          - conan
        targets:
          chat:
            type: tool
            platform: macOS
            sources:
              - src
            configFiles:
              Debug: conan/conan_config.xcconfig
              Release: conan/conan_config.xcconfig
        """)

    client.save({
        "src/main.cpp": '#include "chat.h"\nint main(){chat();return 0;}',
        "project.yml": xcode_project
    }, clean_first=True)

    client.run("install chat/1.0@ -g XcodeDeps --install-folder=conan")
    client.run("install chat/1.0@ -g XcodeDeps --install-folder=conan -s build_type=Debug "
               "--build=missing")
    chat_xcconfig = client.load(os.path.join("conan", "conan_chat_chat.xcconfig"))
    assert '#include "conan_network_client.xcconfig"' in chat_xcconfig
    assert '#include "conan_network_server.xcconfig"' not in chat_xcconfig
    assert '#include "conan_network_network.xcconfig"' not in chat_xcconfig
    host_arch = client.get_default_host_profile().settings['arch']
    arch = "arm64" if host_arch == "armv8" else host_arch
    client.run_command("xcodegen generate")
    client.run_command(f"xcodebuild -project ChatApp.xcodeproj -configuration Release -arch {arch}")
    client.run_command(f"xcodebuild -project ChatApp.xcodeproj -configuration Debug -arch {arch}")
    client.run_command("build/Debug/chat")
    assert "core/1.0: Hello World Debug!" in client.out
    assert "tcp/1.0: Hello World Debug!" in client.out
    assert "client/1.0: Hello World Debug!" in client.out
    assert "chat/1.0: Hello World Debug!" in client.out
    client.run_command("build/Release/chat")
    assert "core/1.0: Hello World Release!" in client.out
    assert "tcp/1.0: Hello World Release!" in client.out
    assert "client/1.0: Hello World Release!" in client.out
    assert "chat/1.0: Hello World Release!" in client.out


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
@pytest.mark.tool_cmake
def test_xcodedeps_test_require():
    client = TestClient()
    client.run("new gtest/1.0 -m cmake_lib")
    # client.run("new cmake_lib -d name=app -d version=1.0")
    client.run("create . -tf=None")

    # Create library having build and test requires
    conanfile = textwrap.dedent(r'''
        from conan import ConanFile
        class HelloLib(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def build_requirements(self):
                self.test_requires('gtest/1.0')
        ''')
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -g XcodeDeps")
    host_arch = client.get_default_host_profile().settings['arch']
    arch = "arm64" if host_arch == "armv8" else host_arch
    assert os.path.isfile(os.path.join(client.current_folder, "conan_gtest.xcconfig"))
    assert os.path.isfile(os.path.join(client.current_folder, "conan_gtest_gtest.xcconfig"))
    assert os.path.isfile(os.path.join(client.current_folder,
                                       f"conan_gtest_gtest_release_{arch}.xcconfig"))
    assert '#include "conan_gtest.xcconfig"' in client.load("conandeps.xcconfig")



import os
import textwrap
import pytest

from conans.client.profile_loader import ProfileParser, read_profile
from conans.errors import ConanException
from conans.model.env_info import EnvValues
from conans.model.profile import Profile
from conans.model.ref import ConanFileReference
from conans.test.utils.mocks import ConanFileMock
from conans.test.utils.profiles import create_profile as _create_profile
from conans.test.utils.test_files import temp_folder
from conans.util.files import load, save


def create_profile(folder, name, settings=None, package_settings=None, env=None,
                   package_env=None, options=None):
    _create_profile(folder, name, settings, package_settings, env, package_env, options)
    content = load(os.path.join(folder, name))
    content = "include(default)\n" + content
    save(os.path.join(folder, name), content)


def get_profile(folder, txt):
    abs_profile_path = os.path.join(folder, "Myprofile.txt")
    save(abs_profile_path, txt)
    return read_profile(abs_profile_path, None, None)


def test_profile_parser():
    txt = textwrap.dedent("""
        include(a/path/to\profile.txt)
        VAR=2
        include(other/path/to/file.txt)
        OTHERVAR=thing

        [settings]
        os=2
        """)
    a = ProfileParser(txt)
    assert a.vars == {"VAR": "2", "OTHERVAR": "thing"}
    assert a.includes == ["a/path/to\profile.txt", "other/path/to/file.txt"]
    assert a.profile_text == textwrap.dedent("""[settings]
os=2""")

    txt = ""
    a = ProfileParser(txt)
    assert a.vars == {}
    assert a.includes == []
    assert a.profile_text == ""

    txt = textwrap.dedent("""
        include(a/path/to\profile.txt)
        VAR=$REPLACE_VAR
        include(other/path/to/$FILE)
        OTHERVAR=thing

        [settings]
        os=$OTHERVAR
        """)

    a = ProfileParser(txt)
    a.update_vars({"REPLACE_VAR": "22", "FILE": "MyFile", "OTHERVAR": "thing"})
    a.apply_vars()
    assert a.vars == {"VAR": "22", "OTHERVAR": "thing",
                      "REPLACE_VAR": "22", "FILE": "MyFile",}
    assert [i for i in a.get_includes()] == ["a/path/to\profile.txt", "other/path/to/MyFile"]
    assert a.profile_text == textwrap.dedent("""[settings]
os=thing""")

    txt = textwrap.dedent("""
        includes(a/path/to\profile.txt)
        """)

    with pytest.raises(ConanException):
        try:
            ProfileParser(txt)
        except Exception as error:
            assert "Error while parsing line 1" in error.args[0]
            raise


def test_profile_loads():

    tmp = temp_folder()

    prof = textwrap.dedent("""
        [env]
        CXX_FLAGS="-DAAA=0"
        [settings]
    """)
    new_profile, _ = get_profile(tmp, prof)
    assert new_profile.env_values.env_dicts("") == ({'CXX_FLAGS': '-DAAA=0'}, {})

    prof = textwrap.dedent("""
        [env]
        CXX_FLAGS="-DAAA=0"
        MyPackage:VAR=1
        MyPackage:OTHER=2
        OtherPackage:ONE=ONE
        [settings]
    """)
    new_profile, _ = get_profile(tmp, prof)
    assert new_profile.env_values.env_dicts("") == ({'CXX_FLAGS': '-DAAA=0'}, {})
    assert new_profile.env_values.env_dicts("MyPackage") == ({"OTHER": "2",
                                                              "VAR": "1",
                                                              "CXX_FLAGS": "-DAAA=0"}, {})

    assert new_profile.env_values.env_dicts("OtherPackage") == ({'CXX_FLAGS': '-DAAA=0',
                                                                 'ONE': 'ONE'}, {})

    prof = textwrap.dedent("""
            [env]
            CXX_FLAGS='-DAAA=0'
            [settings]
        """)
    new_profile, _ = get_profile(tmp, prof)
    assert new_profile.env_values.env_dicts("") == ({'CXX_FLAGS': '-DAAA=0'}, {})

    prof = textwrap.dedent("""
            [env]
            CXX_FLAGS=-DAAA=0
            [settings]
        """)
    new_profile, _ = get_profile(tmp, prof)
    assert new_profile.env_values.env_dicts("") == ({'CXX_FLAGS': '-DAAA=0'}, {})

    prof = textwrap.dedent("""
            [env]
            CXX_FLAGS="-DAAA=0
            [settings]
        """)
    new_profile, _ = get_profile(tmp, prof)
    assert new_profile.env_values.env_dicts("") == ({'CXX_FLAGS': '"-DAAA=0'}, {})

    prof = textwrap.dedent("""
            [settings]
            zlib:compiler=gcc
            compiler=Visual Studio
        """)
    new_profile, _ = get_profile(tmp, prof)
    assert new_profile.package_settings["zlib"] == {"compiler": "gcc"}
    assert new_profile.settings["compiler"] == "Visual Studio"


def test_profile_empty_env():
    tmp = temp_folder()
    profile, _ = get_profile(tmp, "")
    assert isinstance(profile.env_values, EnvValues)


def test_profile_empty_env_settings():
    tmp = temp_folder()
    profile, _ = get_profile(tmp, "[settings]")
    assert isinstance(profile.env_values, EnvValues)


def test_profile_loads_win():
    tmp = temp_folder()
    prof = textwrap.dedent("""
            [env]
            QTPATH=C:/QtCommercial/5.8/msvc2015_64/bin
            QTPATH2="C:/QtCommercial2/5.8/msvc2015_64/bin"
        """)
    new_profile, _ = get_profile(tmp, prof)
    assert new_profile.env_values.data[None]["QTPATH"] == "C:/QtCommercial/5.8/msvc2015_64/bin"
    assert new_profile.env_values.data[None]["QTPATH2"] == "C:/QtCommercial2/5.8/msvc2015_64/bin"
    assert "QTPATH=C:/QtCommercial/5.8/msvc2015_64/bin" in new_profile.dumps()
    assert "QTPATH2=C:/QtCommercial2/5.8/msvc2015_64/bin" in new_profile.dumps()


def test_profile_load_dump():
    # Empty profile
    tmp = temp_folder()
    profile = Profile()
    dumps = profile.dumps()
    new_profile, _ = get_profile(tmp, dumps)
    assert new_profile.settings == profile.settings

    # Settings
    profile = Profile()
    profile.settings["arch"] = "x86_64"
    profile.settings["compiler"] = "Visual Studio"
    profile.settings["compiler.version"] = "12"

    profile.env_values.add("CXX", "path/to/my/compiler/g++")
    profile.env_values.add("CC", "path/to/my/compiler/gcc")

    profile.build_requires["*"] = ["android_toolchain/1.2.8@lasote/testing"]
    profile.build_requires["zlib/*"] = ["cmake/1.0.2@lasote/stable",
                                        "autotools/1.0.3@lasote/stable"]

    dump = profile.dumps()
    new_profile, _ = get_profile(tmp, dump)
    assert new_profile.settings == profile.settings
    assert new_profile.settings["arch"] == "x86_64"
    assert new_profile.settings["compiler.version"] == "12"
    assert new_profile.settings["compiler"] == "Visual Studio"

    assert new_profile.env_values.env_dicts("") == ({'CXX': 'path/to/my/compiler/g++',
                                                     'CC': 'path/to/my/compiler/gcc'}, {})

    assert new_profile.build_requires["zlib/*"] == \
           [ConanFileReference.loads("cmake/1.0.2@lasote/stable"),
            ConanFileReference.loads("autotools/1.0.3@lasote/stable")]
    assert new_profile.build_requires["*"] == \
           [ConanFileReference.loads("android_toolchain/1.2.8@lasote/testing")]


def test_profile_vars():
    tmp = temp_folder()

    txt = textwrap.dedent("""
            MY_MAGIC_VAR=The owls are not

            [env]
            MYVAR=$MY_MAGIC_VAR what they seem.
        """)
    profile, _ = get_profile(tmp, txt)
    assert "The owls are not what they seem." == profile.env_values.data[None]["MYVAR"]

    # Order in replacement, variable names (simplification of preprocessor)
    txt = textwrap.dedent("""
            P=Diane, the coffee at the Great Northern
            P2=is delicious

            [env]
            MYVAR=$P2
        """)
    profile, _ = get_profile(tmp, txt)
    assert "Diane, the coffee at the Great Northern2" == profile.env_values.data[None]["MYVAR"]

    # Variables without spaces
    txt = textwrap.dedent("""
            VARIABLE WITH SPACES=12
            [env]
            MYVAR=$VARIABLE WITH SPACES
        """)
    with pytest.raises(ConanException) as conan_error:
        get_profile(tmp, txt)
        assert "The names of the variables cannot contain spaces" in str(conan_error.value)


def test_profiles_includes():
    tmp = temp_folder()

    def save_profile(txt, name):
        abs_profile_path = os.path.join(tmp, name)
        save(abs_profile_path, txt)

    os.mkdir(os.path.join(tmp, "subdir"))

    profile0 = textwrap.dedent("""
            ROOTVAR=0


            [build_requires]
            one/1.$ROOTVAR@lasote/stable
            two/1.2@lasote/stable
        """)
    save_profile(profile0, "subdir/profile0.txt")

    profile1 = textwrap.dedent("""
            # Include in subdir, curdir
            MYVAR=1
            include(./profile0.txt)

            [settings]
            os=Windows
            [options]
            zlib:aoption=1
            zlib:otheroption=1
            [env]
            package1:ENVY=$MYVAR
        """)

    save_profile(profile1, "subdir/profile1.txt")

    profile2 = textwrap.dedent("""
            #  Include in subdir
            include(./subdir/profile1.txt)
            [settings]
            os=$MYVAR
        """)

    save_profile(profile2, "profile2.txt")
    profile3 = textwrap.dedent("""
            OTHERVAR=34

            [build_requires]
            one/1.5@lasote/stable
        """)

    save_profile(profile3, "profile3.txt")

    profile4 = textwrap.dedent("""
            include(./profile2.txt)
            include(./profile3.txt)

            [env]
            MYVAR=FromProfile3And$OTHERVAR

            [options]
            zlib:otheroption=12
        """)

    save_profile(profile4, "profile4.txt")

    profile, variables = read_profile("./profile4.txt", tmp, None)

    assert variables == {"MYVAR": "1",
                         "OTHERVAR": "34",
                         "PROFILE_DIR": tmp.replace('\\', '/'),
                         "ROOTVAR": "0"}
    assert "FromProfile3And34" == profile.env_values.data[None]["MYVAR"]
    assert "1" == profile.env_values.data["package1"]["ENVY"]
    assert profile.settings == {"os": "1"}
    assert profile.options.as_list() == [('zlib:aoption', '1'), ('zlib:otheroption', '12')]
    assert profile.build_requires == {"*": [ConanFileReference.loads("one/1.5@lasote/stable"),
                                            ConanFileReference.loads("two/1.2@lasote/stable")]}


def test_profile_dir():
    tmp = temp_folder()
    txt = textwrap.dedent("""
            [env]
            PYTHONPATH=$PROFILE_DIR/my_python_tools
        """)

    def assert_path(profile):
        pythonpath = profile.env_values.env_dicts("")[0]["PYTHONPATH"]
        assert pythonpath == os.path.join(tmp, "my_python_tools").replace('\\', '/')

    abs_profile_path = os.path.join(tmp, "Myprofile.txt")
    save(abs_profile_path, txt)
    profile, _ = read_profile(abs_profile_path, None, None)
    assert_path(profile)

    profile, _ = read_profile("./Myprofile.txt", tmp, None)
    assert_path(profile)

    profile, _ = read_profile("Myprofile.txt", None, tmp)
    assert_path(profile)


def test_profile_include_order():
    tmp = temp_folder()

    save(os.path.join(tmp, "profile1.txt"), "MYVAR=fromProfile1")

    profile2 = textwrap.dedent("""
            include(./profile1.txt)
            MYVAR=fromProfile2
            [settings]
            os=$MYVAR
        """)
    save(os.path.join(tmp, "profile2.txt"), profile2)
    profile, variables = read_profile("./profile2.txt", tmp, None)

    assert variables == {"MYVAR": "fromProfile2",
                         "PROFILE_DIR": tmp.replace('\\', '/')}
    assert profile.settings["os"] == "fromProfile2"


def test_profile_load_absolute_path():
    """ When passing absolute path as profile file, it MUST be used.
        read_profile(/abs/path/profile, /abs, /.conan/profiles)
        /abs/path/profile MUST be consumed as target profile
    """
    profile_name = "default"
    default_profile_folder = os.path.join(temp_folder(), "profiles")
    default_profile_path = os.path.join(default_profile_folder, profile_name)
    current_profile_folder = temp_folder()
    current_profile_path = os.path.join(current_profile_folder, profile_name)
    default_profile_content = textwrap.dedent("""
            [env]
            BORSCHT=BEET SOUP
        """)
    current_profile_content = default_profile_content.replace("BEET", "RUSSIAN")

    save(default_profile_path, default_profile_content)
    save(current_profile_path, current_profile_content)

    profile, variables = read_profile(current_profile_path, current_profile_folder,
                                      default_profile_folder)
    assert ({"BORSCHT": "RUSSIAN SOUP"}, {}) == profile.env_values.env_dicts("")
    assert current_profile_folder.replace("\\", "/") == variables["PROFILE_DIR"]


def test_profile_load_relative_path_dot():
    """ When passing relative ./path as profile file, it MUST be used
        read_profile(./profiles/profile, /tmp, /.conan/profiles)
        /tmp/profiles/profile MUST be consumed as target profile
    """
    profile_name = "default"
    default_profile_folder = os.path.join(temp_folder(), "profiles")
    default_profile_path = os.path.join(default_profile_folder, profile_name)
    current_profile_folder = temp_folder()
    current_profile_path = os.path.join(current_profile_folder, profile_name)
    default_profile_content = textwrap.dedent("""
            [env]
            BORSCHT=BEET SOUP
        """)
    current_profile_content = default_profile_content.replace("BEET", "RUSSIAN")
    relative_current_profile_path = "." + os.path.join(os.sep,
                                                       os.path.basename(current_profile_folder),
                                                       profile_name)

    save(default_profile_path, default_profile_content)
    save(current_profile_path, current_profile_content)

    profile, variables = read_profile(relative_current_profile_path,
                                      os.path.dirname(current_profile_folder),
                                      default_profile_folder)
    assert ({"BORSCHT": "RUSSIAN SOUP"}, {}) == profile.env_values.env_dicts("")
    assert current_profile_folder.replace("\\", "/") == variables["PROFILE_DIR"]


def test_profile_load_relative_path_pardir():
    """ When passing relative ../path as profile file, it MUST be used
        read_profile(../profiles/profile, /tmp/current, /.conan/profiles)
        /tmp/profiles/profile MUST be consumed as target profile
    """
    profile_name = "default"
    default_profile_folder = os.path.join(temp_folder(), "profiles")
    os.mkdir(default_profile_folder)
    default_profile_path = os.path.join(default_profile_folder, profile_name)

    current_temp_folder = temp_folder()
    current_profile_folder = os.path.join(current_temp_folder, "profiles")
    current_running_folder = os.path.join(current_temp_folder, "current")

    os.mkdir(current_profile_folder)
    os.mkdir(current_running_folder)

    current_profile_path = os.path.join(current_profile_folder, profile_name)
    default_profile_content = textwrap.dedent("""
            [env]
            BORSCHT=BEET SOUP
        """)
    current_profile_content = default_profile_content.replace("BEET", "RUSSIAN")
    relative_current_profile_path = os.pardir + os.path.join(os.sep, "profiles", profile_name)

    save(default_profile_path, default_profile_content)
    save(current_profile_path, current_profile_content)

    profile, variables = read_profile(relative_current_profile_path,
                                      current_running_folder,
                                      default_profile_folder)
    assert ({"BORSCHT": "RUSSIAN SOUP"}, {}) == profile.env_values.env_dicts("")
    assert current_profile_folder.replace("\\", "/") == variables["PROFILE_DIR"]


def test_profile_buildenv():
    tmp = temp_folder()
    txt = textwrap.dedent("""
        [buildenv]
        MyVar1=My Value; 11
        MyVar1+=MyValue12
        MyPath1=(path)/some/path11
        MyPath1+=(path)/other path/path12
        """)
    profile, _ = get_profile(tmp, txt)
    buildenv = profile.buildenv
    env = buildenv.get_profile_env(None)
    conanfile = ConanFileMock()
    env_vars = env.vars(conanfile)
    assert env_vars.get("MyVar1") == "My Value; 11 MyValue12"
    assert env_vars.get("MyPath1") == "/some/path11{}/other path/path12".format(os.pathsep)


def test_profile_tool_requires():
    tmp = temp_folder()
    txt = textwrap.dedent("""
        [tool_requires]
        tool/0.1
        """)
    profile, _ = get_profile(tmp, txt)
    assert profile.build_requires["*"] == [ConanFileReference.loads("tool/0.1")]



import os
import unittest
import uuid

import six
import pytest

from conans.client.store.localdb import LocalDB
from conans.test.utils.test_files import temp_folder


class LocalStoreTest(unittest.TestCase):

    def test_localdb(self):
        tmp_dir = temp_folder()
        db_file = os.path.join(tmp_dir, "dbfile")
        localdb = LocalDB.create(db_file)

        # Test write and read login
        user, token, access_token = localdb.get_login("myurl1")
        self.assertIsNone(user)
        self.assertIsNone(token)
        self.assertIsNone(access_token)

        localdb.store("pepe", "token", "access_token", "myurl1")
        user, token, access_token = localdb.get_login("myurl1")
        self.assertEqual("pepe", user)
        self.assertEqual("token", token)
        self.assertEqual("access_token", access_token)
        self.assertEqual("pepe", localdb.get_username("myurl1"))

    def test_token_encryption_ascii(self):
        tmp_dir = temp_folder()
        db_file = os.path.join(tmp_dir, "dbfile")
        encryption_key = str(uuid.uuid4())
        localdb = LocalDB.create(db_file, encryption_key=encryption_key)

        localdb.store("pepe", "token", "access_token", "myurl1")
        user, token, access_token = localdb.get_login("myurl1")
        self.assertEqual("pepe", user)
        self.assertEqual("token", token)
        self.assertEqual("access_token", access_token)

    def test_token_encryption_none(self):
        tmp_dir = temp_folder()
        db_file = os.path.join(tmp_dir, "dbfile")
        encryption_key = str(uuid.uuid4())
        localdb = LocalDB.create(db_file, encryption_key=encryption_key)

        localdb.store("pepe", "token", None, "myurl1")
        user, token, access_token = localdb.get_login("myurl1")
        self.assertEqual("pepe", user)
        self.assertEqual("token", token)
        self.assertEqual(None, access_token)

    @pytest.mark.skipif(six.PY2, reason="Python2 sqlite3 converts to str")
    def test_token_encryption_unicode(self):
        tmp_dir = temp_folder()
        db_file = os.path.join(tmp_dir, "dbfile")
        encryption_key = str(uuid.uuid4())
        localdb = LocalDB.create(db_file, encryption_key=encryption_key)

        token_input = b'espa\xc3\xb1a\xe2\x82\xac$'.decode('utf-8')  # Only ASCII files in codebase
        localdb.store("pepe", token_input, token_input, "myurl1")
        user, token, access_token = localdb.get_login("myurl1")
        self.assertEqual("pepe", user)
        self.assertEqual(token_input, token)
        self.assertEqual(token_input, access_token)
        self.assertEqual("pepe", localdb.get_username("myurl1"))

        # Without the encryption key we get obfuscated values
        other_db = LocalDB.create(db_file)
        user, token, access_token = other_db.get_login("myurl1")
        self.assertEqual("pepe", user)
        self.assertNotEqual(token_input, token)
        self.assertNotEqual(token_input, access_token)



import os

from conans.model.new_build_info import NewCppInfo
from conans.model.layout import Folders as Folders_co, Infos as Infos_co


class Infos(Infos_co):

    def __init__(self):
        super().__init__()
        self.deb = NewCppInfo(with_defaults=True)

class Folders(Folders_co):
    
    def __init__(self):
        super().__init__()
        self._base_deb = None
        self.deb = ""

    @property
    def base_deb(self):
        return self._base_deb

    def set_base_deb(self, folder):
        self._base_deb = folder

    @property
    def deb_folder(self):
        """For the cache, the package folder is only the base"""
        return self._base_deb



import os

from conans.model.ref import ConanFileReference, PackageReference
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.test_files import uncompress_packaged_files
from conans.test.utils.tools import TestClient


def test_reuse_uploaded_tgz():
    client = TestClient(default_server_user=True)
    # Download packages from a remote, then copy to another channel
    # and reupload them. Because they have not changed, the tgz is not created again

    # UPLOAD A PACKAGE
    ref = ConanFileReference.loads("Hello0/0.1@user/stable")
    files = {"conanfile.py": GenConanfile("Hello0", "0.1").with_exports("*"),
             "another_export_file.lib": "to compress"}
    client.save(files)
    client.run("create . user/stable")
    client.run("upload %s --all" % str(ref))
    assert "Compressing recipe" in client.out
    assert "Compressing package" in client.out

    # UPLOAD TO A DIFFERENT CHANNEL WITHOUT COMPRESS AGAIN
    client.run("copy %s user/testing --all" % str(ref))
    client.run("upload Hello0/0.1@user/testing --all")
    assert "Compressing recipe" not in client.out
    assert "Compressing package" not in client.out


def test_reuse_downloaded_tgz():
    # Download packages from a remote, then copy to another channel
    # and reupload them. It needs to compress it again, not tgz is kept
    client = TestClient(default_server_user=True)
    # UPLOAD A PACKAGE
    files = {"conanfile.py": GenConanfile("Hello0", "0.1").with_exports("*"),
             "another_export_file.lib": "to compress"}
    client.save(files)
    client.run("create . user/stable")
    client.run("upload Hello0/0.1@user/stable --all")
    assert "Compressing recipe" in client.out
    assert "Compressing package" in client.out

    # Other user downloads the package
    # THEN A NEW USER DOWNLOADS THE PACKAGES AND UPLOADS COMPRESSING AGAIN
    # BECAUSE ONLY TGZ IS KEPT WHEN UPLOADING
    other_client = TestClient(servers=client.servers, users={"default": [("user", "password")]})
    other_client.run("download Hello0/0.1@user/stable")
    other_client.run("upload Hello0/0.1@user/stable --all")
    assert "Compressing recipe" in client.out
    assert "Compressing package" in client.out


def test_upload_only_tgz_if_needed():
    client = TestClient(default_server_user=True)
    ref = ConanFileReference.loads("Hello0/0.1@user/stable")
    conanfile = GenConanfile("Hello0", "0.1").with_exports("*").with_package_file("lib/file.lib",
                                                                                  "File")
    client.save({"conanfile.py": conanfile,
                 "file.txt": "contents"})
    client.run("create . user/stable")

    # Upload conans
    client.run("upload %s" % str(ref))
    assert "Compressing recipe" in client.out

    # Not needed to tgz again
    client.run("upload %s" % str(ref))
    assert "Compressing recipe" not in client.out

    # Check that conans exists on server
    server_paths = client.servers["default"].server_store
    conan_path = server_paths.conan_revisions_root(ref)
    assert os.path.exists(conan_path)
    package_ids = client.cache.package_layout(ref).package_ids()
    pref = PackageReference(ref, package_ids[0])

    # Upload package
    client.run("upload %s -p %s" % (str(ref), str(package_ids[0])))
    assert "Compressing package" in client.out

    # Not needed to tgz again
    client.run("upload %s -p %s" % (str(ref), str(package_ids[0])))
    assert "Compressing package" not in client.out

    # If we install the package again will be removed and re tgz
    client.run("install %s --build missing" % str(ref))
    # Upload package
    client.run("upload %s -p %s" % (str(ref), str(package_ids[0])))
    assert "Compressing package" not in client.out

    # Check library on server
    folder = uncompress_packaged_files(server_paths, pref)
    libraries = os.listdir(os.path.join(folder, "lib"))
    assert len(libraries) == 1



import unittest
from collections import namedtuple

from mock import Mock

from conans.client.cache.remote_registry import Remote
from conans.client.rest.auth_manager import ConanApiAuthManager
from conans.client.rest.rest_client import RestApiClientFactory
from conans.model.ref import ConanFileReference
from conans.test.utils.mocks import LocalDBMock, TestBufferConanOutput
from conans.client.userio import UserIO


common_headers = {"X-Conan-Server-Capabilities": "oauth_token", "Content-Type": "application/json"}


class ResponseOK(object):
    def __init__(self):
        self.ok = True
        self.headers = common_headers
        self.status_code = 200
        self.content = b''


class ResponseDownloadURLs(object):
    def __init__(self):
        self.ok = True
        self.headers = common_headers
        self.status_code = 200
        self.content = b'{}'


class ResponseWithTokenMock(object):
    def __init__(self, token):
        self.token = token
        self.ok = True
        self.headers = common_headers
        self.status_code = 200
        self.content = ''

    def json(self):
        return {"access_token": self.token, "refresh_token": "refresh_token"}


class ResponseAuthenticationRequired(object):
    def __init__(self):
        self.ok = False
        self.headers = common_headers
        self.status_code = 401
        self.content = b'Login needed'


class RequesterWithTokenMock(object):

    def get(self, url, **kwargs):
        if not kwargs["auth"].token or kwargs["auth"].token == "expired":
            return ResponseAuthenticationRequired()
        if url.endswith("download_urls"):
            return ResponseDownloadURLs()
        elif url.endswith("users/authenticate"):
            raise Exception("This endpoint should't be called when oauth supported")

    def post(self, url, **kwargs):
        """If the call is to refresh we return "refreshed_access_token" otherwise we return
        "access_token"
        """
        if url.endswith("users/token"):
            if kwargs.get("data") and kwargs.get("data").get("grant_type") == "refresh_token":
                return ResponseWithTokenMock("refreshed_access_token")
            return ResponseWithTokenMock("access_token")
        else:
            raise Exception("This endpoint should't be reached")


class TestTokenRefresh(unittest.TestCase):

    def setUp(self):
        mocked_user_io = UserIO(out=TestBufferConanOutput())
        mocked_user_io.get_username = Mock(return_value="myuser")
        mocked_user_io.get_password = Mock(return_value="mypassword")

        requester = RequesterWithTokenMock()
        config = namedtuple("ConfigMock", "revisions_enabled download_cache retry retry_wait")\
            (False, None, None, None)
        self.rest_client_factory = RestApiClientFactory(mocked_user_io.out,
                                                        requester, config=config,
                                                        artifacts_properties=None)
        self.localdb = LocalDBMock()
        self.auth_manager = ConanApiAuthManager(self.rest_client_factory, mocked_user_io,
                                                self.localdb)
        self.remote = Remote("myremote", "myurl", True, True)
        self.ref = ConanFileReference.loads("lib/1.0@conan/stable")

    def test_auth_with_token(self):
        """Test that if the capability is there, then we use the new endpoint"""
        self.auth_manager.call_rest_api_method(self.remote, "get_recipe", self.ref, ".")
        self.assertEqual(self.localdb.user, "myuser")
        self.assertEqual(self.localdb.access_token, "access_token")
        self.assertEqual(self.localdb.refresh_token, "refresh_token")

    def test_refresh_with_token(self):
        """The mock will raise 401 for a token value "expired" so it will try to refresh
        and only if the refresh endpoint is called, the value will be "refreshed_access_token"
        """
        self.localdb.access_token = "expired"
        self.localdb.refresh_token = "refresh_token"

        self.auth_manager.call_rest_api_method(self.remote, "get_recipe", self.ref, ".")
        self.assertEqual(self.localdb.user, "myuser")
        self.assertEqual(self.localdb.access_token, "refreshed_access_token")
        self.assertEqual(self.localdb.refresh_token, "refresh_token")



import os
import platform
import textwrap
import unittest

import pytest
import six
import mock
from parameterized import parameterized

from conans.client import tools
from conans.client.tools.files import replace_in_file
from conans.model.ref import PackageReference
from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient
from conans.test.assets.visual_project_files import get_vs_project_files
from conans.util.files import load
from conans.client.tools.files import chdir
from conans.client.build.msbuild import MSBuild
from conans.errors import ConanException
from conans.model.version import Version
from conans.test.utils.mocks import MockSettings, MockConanfile, ConanFileMock
from conans.test.utils.test_files import temp_folder


class MSBuildTest(unittest.TestCase):

    @pytest.mark.slow
    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows" or six.PY2, reason="Requires MSBuild")
    def test_build_vs_project(self):
        conan_build_vs = """
from conans import ConanFile, MSBuild

class HelloConan(ConanFile):
    name = "Hello"
    version = "1.2.1"
    exports = "*"
    settings = "os", "build_type", "arch", "compiler", "cppstd"

    def build(self):
        msbuild = MSBuild(self)
        msbuild.build("MyProject.sln", verbosity="normal")

    def package(self):
        self.copy(pattern="*.exe")
"""
        client = TestClient()

        # Test cpp standard stuff

        files = get_vs_project_files(std="cpp17_2015")
        files[CONANFILE] = conan_build_vs

        client.save(files)
        client.run('create . Hello/1.2.1@lasote/stable -s cppstd=11 -s '
                   'compiler="Visual Studio" -s compiler.version=14', assert_error=True)

        client.run('create . Hello/1.2.1@lasote/stable -s cppstd=17 '
                   '-s compiler="Visual Studio" -s compiler.version=14')
        self.assertIn("Packaged 1 '.exe' file: MyProject.exe", client.out)

        files = get_vs_project_files()
        files[CONANFILE] = conan_build_vs

        # Try to not update the project
        client.cache._config = None  # Invalidate cached config

        replace_in_file(client.cache.conan_conf_path, "[general]",
                        "[general]\nskip_vs_projects_upgrade = True", output=client.out)
        client.save(files, clean_first=True)
        client.run("create . Hello/1.2.1@lasote/stable --build")
        self.assertNotIn("devenv", client.out)
        self.assertIn("Skipped sln project upgrade", client.out)

        # Try with x86_64
        client.save(files)
        client.run("export . lasote/stable")
        client.run("install Hello/1.2.1@lasote/stable --build -s arch=x86_64")
        self.assertIn("Release|x64", client.out)
        self.assertIn("Packaged 1 '.exe' file: MyProject.exe", client.out)

        # Try with x86
        client.save(files, clean_first=True)
        client.run("export . lasote/stable")
        client.run("install Hello/1.2.1@lasote/stable --build -s arch=x86")
        self.assertIn("Release|x86", client.out)
        self.assertIn("Packaged 1 '.exe' file: MyProject.exe", client.out)

        # Try with x86 debug
        client.save(files, clean_first=True)
        client.run("export . lasote/stable")
        client.run("install Hello/1.2.1@lasote/stable --build -s arch=x86 -s build_type=Debug")
        self.assertIn("Debug|x86", client.out)
        self.assertIn("Packaged 1 '.exe' file: MyProject.exe", client.out)

        # Try with a custom property file name
        files[CONANFILE] = conan_build_vs.replace(
                'msbuild.build("MyProject.sln", verbosity="normal")',
                'msbuild.build("MyProject.sln", verbosity="normal", property_file_name="mp.props")')
        client.save(files, clean_first=True)
        client.run("create . Hello/1.2.1@lasote/stable --build -s arch=x86 -s build_type=Debug")
        self.assertIn("Debug|x86", client.out)
        self.assertIn("Packaged 1 '.exe' file: MyProject.exe", client.out)
        full_ref = "Hello/1.2.1@lasote/stable:b786e9ece960c3a76378ca4d5b0d0e922f4cedc1"
        pref = PackageReference.loads(full_ref)
        build_folder = client.cache.package_layout(pref.ref).build(pref)
        self.assertTrue(os.path.exists(os.path.join(build_folder, "mp.props")))

    @pytest.mark.slow
    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    def test_user_properties_file(self):
        conan_build_vs = textwrap.dedent("""
            from conans import ConanFile, MSBuild

            class HelloConan(ConanFile):
                exports = "*"
                settings = "os", "build_type", "arch", "compiler"

                def build(self):
                    msbuild = MSBuild(self)
                    msbuild.build("MyProject.sln", verbosity="normal",
                                  definitions={"MyCustomDef": "MyCustomValue"},
                                  user_property_file_name="myuser.props")

                def package(self):
                    self.copy(pattern="*.exe")
            """)
        client = TestClient()

        files = get_vs_project_files()
        files[CONANFILE] = conan_build_vs
        props = textwrap.dedent("""<?xml version="1.0" encoding="utf-8"?>
            <Project ToolsVersion="4.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
              <ImportGroup Label="PropertySheets" />
              <PropertyGroup Label="UserMacros" />
              <ItemDefinitionGroup>
                <ClCompile>
                  <RuntimeLibrary>MultiThreaded</RuntimeLibrary>
                </ClCompile>
              </ItemDefinitionGroup>
              <ItemGroup />
            </Project>
            """)
        files["myuser.props"] = props

        client.save(files)
        client.run('create . Hello/1.2.1@lasote/stable')
        self.assertNotIn("/EHsc /MD", client.out)
        self.assertIn("/EHsc /MT", client.out)
        self.assertIn("/D MyCustomDef=MyCustomValue", client.out)
        self.assertIn("Packaged 1 '.exe' file: MyProject.exe", client.out)

        full_ref = "Hello/1.2.1@lasote/stable:6cc50b139b9c3d27b3e9042d5f5372d327b3a9f7"
        pref = PackageReference.loads(full_ref)
        build_folder = client.cache.package_layout(pref.ref).build(pref)
        self.assertTrue(os.path.exists(os.path.join(build_folder, "myuser.props")))
        conan_props = os.path.join(build_folder, "conan_build.props")
        content = load(conan_props)
        self.assertIn("<RuntimeLibrary>MultiThreadedDLL</RuntimeLibrary>", content)

    @pytest.mark.slow
    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    def test_user_properties_multifile(self):
        conan_build_vs = textwrap.dedent("""
            from conans import ConanFile, MSBuild

            class HelloConan(ConanFile):
                exports = "*"
                settings = "os", "build_type", "arch", "compiler"

                def build(self):
                    msbuild = MSBuild(self)
                    msbuild.build("MyProject.sln", verbosity="normal",
                                  definitions={"MyCustomDef": "MyCustomValue"},
                                  user_property_file_name=["myuser.props", "myuser2.props"])

                def package(self):
                    self.copy(pattern="*.exe")
            """)
        client = TestClient()

        files = get_vs_project_files()
        files[CONANFILE] = conan_build_vs
        props = textwrap.dedent("""<?xml version="1.0" encoding="utf-8"?>
            <Project ToolsVersion="4.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
              <ImportGroup Label="PropertySheets" />
              <PropertyGroup Label="UserMacros" />
              <ItemDefinitionGroup>
                <ClCompile>
                    <PreprocessorDefinitions>MyCustomDef2=MyValue2;%(PreprocessorDefinitions)
                    </PreprocessorDefinitions>
                </ClCompile>
              </ItemDefinitionGroup>
              <ItemGroup />
            </Project>
            """)
        props2 = textwrap.dedent("""<?xml version="1.0" encoding="utf-8"?>
            <Project ToolsVersion="4.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
              <ImportGroup Label="PropertySheets" />
              <PropertyGroup Label="UserMacros" />
              <ItemDefinitionGroup>
                <ClCompile>
                  <RuntimeLibrary>MultiThreaded</RuntimeLibrary>
                </ClCompile>
              </ItemDefinitionGroup>
              <ItemGroup />
            </Project>
            """)
        files["myuser.props"] = props
        files["myuser2.props"] = props2

        client.save(files)
        client.run('create . Hello/1.2.1@lasote/stable')
        self.assertNotIn("/EHsc /MD", client.out)
        self.assertIn("/EHsc /MT", client.out)
        self.assertIn("/D MyCustomDef=MyCustomValue", client.out)
        self.assertIn("/D MyCustomDef2=MyValue2", client.out)
        self.assertIn("Packaged 1 '.exe' file: MyProject.exe", client.out)

        full_ref = "Hello/1.2.1@lasote/stable:6cc50b139b9c3d27b3e9042d5f5372d327b3a9f7"
        pref = PackageReference.loads(full_ref)
        build_folder = client.cache.package_layout(pref.ref).build(pref)
        self.assertTrue(os.path.exists(os.path.join(build_folder, "myuser.props")))
        conan_props = os.path.join(build_folder, "conan_build.props")
        content = load(conan_props)
        self.assertIn("<RuntimeLibrary>MultiThreadedDLL</RuntimeLibrary>", content)

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    def test_reuse_msbuild_object(self):
        # https://github.com/conan-io/conan/issues/2865
        conan_build_vs = """
from conans import ConanFile, MSBuild

class HelloConan(ConanFile):
    name = "Hello"
    version = "1.2.1"
    exports = "*"
    settings = "os", "build_type", "arch", "compiler", "cppstd"

    def configure(self):
        del self.settings.compiler.runtime
        del self.settings.build_type

    def build(self):
        msbuild = MSBuild(self)
        msbuild.build("MyProject.sln", build_type="Release")
        msbuild.build("MyProject.sln", build_type="Debug")
        self.output.info("build() completed")
"""
        client = TestClient()
        files = get_vs_project_files()
        files[CONANFILE] = conan_build_vs

        client.save(files)
        client.run("create . danimtb/testing")
        self.assertIn("build() completed", client.out)

    @parameterized.expand([("True",), ("'my_log.binlog'",)])
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    @pytest.mark.tool_visual_studio
    def test_binary_log_build(self, value):
        conan_build_vs = """
from conans import ConanFile, MSBuild

class HelloConan(ConanFile):
    name = "Hello"
    version = "1.2.1"
    exports = "*"
    settings = "os", "build_type", "arch", "compiler"

    def build(self):
        msbuild = MSBuild(self)
        msbuild.build("MyProject.sln", output_binary_log=%s)
"""
        client = TestClient()
        files = get_vs_project_files()
        files[CONANFILE] = conan_build_vs % value
        client.save(files)
        client.run("install . -s compiler=\"Visual Studio\" -s compiler.version=15")
        client.run("build .")

        if value == "'my_log.binlog'":
            log_name = value[1:1]
            flag = "/bl:%s" % log_name
        else:
            log_name = "msbuild.binlog"
            flag = "/bl"

        self.assertIn(flag, client.out)
        log_path = os.path.join(client.current_folder, log_name)
        self.assertTrue(os.path.exists(log_path))

    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    @pytest.mark.tool_visual_studio
    def test_binary_logging_on(self):
        settings = MockSettings({"build_type": "Debug",
                                 "compiler": "Visual Studio",
                                 "compiler.version": "15",
                                 "arch": "x86_64",
                                 "compiler.runtime": "MDd"})
        conanfile = MockConanfile(settings)
        msbuild = MSBuild(conanfile)
        command = msbuild.get_command("dummy.sln", output_binary_log=True)
        self.assertIn("/bl", command)

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    def test_binary_logging_on_with_filename(self):
        bl_filename = "a_special_log.log"
        settings = MockSettings({"build_type": "Debug",
                                 "compiler": "Visual Studio",
                                 "compiler.version": "15",
                                 "arch": "x86_64",
                                 "compiler.runtime": "MDd"})
        conanfile = MockConanfile(settings)
        msbuild = MSBuild(conanfile)
        command = msbuild.get_command("dummy.sln", output_binary_log=bl_filename)
        expected_command = '/bl:"%s"' % bl_filename
        self.assertIn(expected_command, command)

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    @mock.patch("conans.client.build.msbuild.MSBuild.get_version")
    def test_binary_logging_not_supported(self, mock_get_version):
        mock_get_version.return_value = Version("14")

        mocked_settings = MockSettings({"build_type": "Debug",
                                        "compiler": "Visual Studio",
                                        "compiler.version": "15",
                                        "arch": "x86_64",
                                        "compiler.runtime": "MDd"})
        conanfile = MockConanfile(mocked_settings)
        except_text = "MSBuild version detected (14) does not support 'output_binary_log' ('/bl')"
        msbuild = MSBuild(conanfile)

        with self.assertRaises(ConanException) as exc:
            msbuild.get_command("dummy.sln", output_binary_log=True)
        self.assertIn(except_text, str(exc.exception))

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    def test_get_version(self):
        settings = MockSettings({"build_type": "Debug",
                                 "compiler": "Visual Studio",
                                 "compiler.version": "15",
                                 "arch": "x86_64",
                                 "compiler.runtime": "MDd"})
        version = MSBuild.get_version(settings)
        six.assertRegex(self, version, r"(\d+\.){2,3}\d+")
        self.assertGreater(version, "15.1")

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
    def test_skip_toolset(self):
        settings = MockSettings({"build_type": "Debug",
                                 "compiler": "Visual Studio",
                                 "compiler.version": "15",
                                 "arch": "x86_64"})

        class Runner(object):

            def __init__(self):
                self.commands = []

            def __call__(self, *args, **kwargs):
                self.commands.append(args[0])

        with chdir(tools.mkdir_tmp()):
            runner = Runner()
            conanfile = MockConanfile(settings, runner=runner)
            msbuild = MSBuild(conanfile)
            msbuild.build("myproject", toolset=False)
            self.assertEqual(len(runner.commands), 1)
            self.assertNotIn("PlatformToolset", runner.commands[0])

            runner = Runner()
            conanfile = MockConanfile(settings, runner=runner)
            msbuild = MSBuild(conanfile)
            msbuild.build("myproject", toolset="mytoolset")
            self.assertEqual(len(runner.commands), 1)
            self.assertIn('/p:PlatformToolset="mytoolset"', runner.commands[0])

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows",
                        reason="Requires Visual Studio installation path")
    def test_arch_override(self):
        settings = MockSettings({"build_type": "Release",
                                 "compiler": "Visual Studio",
                                 "compiler.version": "15",
                                 "compiler.runtime": "MDd",
                                 "os": "Windows",
                                 "arch": "x86_64"})
        conanfile = ConanFileMock()
        conanfile.settings = settings
        props_file_path = os.path.join(temp_folder(), "conan_build.props")

        msbuild = MSBuild(conanfile)
        msbuild.build("project_file.sln", property_file_name=props_file_path)
        self.assertIn("vcvarsall.bat\" amd64", conanfile.command)
        self.assertIn("/p:Platform=\"x64\"", conanfile.command)
        msbuild.build("project_file.sln", arch="x86", property_file_name=props_file_path)
        self.assertIn("vcvarsall.bat\" x86", conanfile.command)
        self.assertIn("/p:Platform=\"x86\"", conanfile.command)



import textwrap

import pytest

from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.pkg_cmake import pkg_cmake
from conans.test.assets.sources import gen_function_h, gen_function_cpp
from conans.test.utils.tools import TestClient


@pytest.fixture(scope="module")
def client_weird_lib_name():
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os, platform
        from conans import ConanFile
        from conan.tools.cmake import CMake, cmake_layout

        class Pkg(ConanFile):
            exports_sources = "CMakeLists.txt", "src/*"
            settings = "os", "compiler", "arch", "build_type"
            generators = "CMakeToolchain", "CMakeDeps"

            def layout(self):
                cmake_layout(self)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()

            def package(self):
                self.copy("*.h", dst="include", src="src")
                self.copy("*.lib", dst="lib", keep_path=False)
                self.copy("*.a", dst="lib", keep_path=False)
                ext = "a" if platform.system() != "Windows" else "lib"
                prefix = "lib" if platform.system() != "Windows" else ""
                os.chdir(os.path.join(self.package_folder, "lib"))
                os.rename("{}hello_0.1.{}".format(prefix, ext),
                          "{}he!llo@0.1.{}".format(prefix, ext))

            def package_info(self):
                self.cpp_info.libs = ["he!llo@0.1"]
            """)

    hdr = gen_function_h(name="hello")
    src = gen_function_cpp(name="hello")
    cmake = gen_cmakelists(libname="hello_0.1", libsources=["src/hello.cpp"])

    c.save({"src/hello.h": hdr,
            "src/hello.cpp": src,
            "CMakeLists.txt": cmake,
            "conanfile.py": conanfile})
    c.run("create . hello/0.1@")
    return c


@pytest.mark.tool_cmake
def test_cmakedeps(client_weird_lib_name):
    c = client_weird_lib_name
    c.save(pkg_cmake("chat", "0.1", requires=["hello/0.1"]), clean_first=True)
    c.run("create . chat/0.1@")
    assert "chat/0.1: Created package" in c.out


# TODO: Remove in Conan 2.0
@pytest.mark.tool_cmake
def test_cmake_find_package(client_weird_lib_name):
    c = client_weird_lib_name
    files = pkg_cmake("chat", "0.1", requires=["hello/0.1"])
    conanfile = textwrap.dedent("""
        import os, platform
        from conans import ConanFile, CMake

        class Pkg(ConanFile):
            exports_sources = "CMakeLists.txt", "src/*", "include/*"
            settings = "os", "compiler", "arch", "build_type"
            generators = "cmake_find_package"
            requires = "hello/0.1"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
        """)
    files["conanfile.py"] = conanfile
    c.save(files, clean_first=True)
    c.run("create . chat/0.1@")
    assert "chat/0.1: Created package" in c.out


# TODO: Remove in Conan 2.0
@pytest.mark.tool_cmake
def test_cmake_find_package_multi(client_weird_lib_name):
    c = client_weird_lib_name
    files = pkg_cmake("chat", "0.1", requires=["hello/0.1"])
    conanfile = textwrap.dedent("""
        import os, platform
        from conans import ConanFile, CMake

        class Pkg(ConanFile):
            exports_sources = "CMakeLists.txt", "src/*", "include/*"
            settings = "os", "compiler", "arch", "build_type"
            generators = "cmake_find_package_multi"
            requires = "hello/0.1"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
        """)
    files["conanfile.py"] = conanfile
    c.save(files, clean_first=True)
    c.run("create . chat/0.1@")
    assert "chat/0.1: Created package" in c.out



import textwrap

from conans.test.utils.tools import TestClient


def test_version():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.scm import Version

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            settings = "compiler"
            def configure(self):
                v = Version(self.settings.compiler.version)
                assert v > "10"

                v = Version("1.2.3")
                self.output.warning(f"The major of 1.2.3 is {v.major}")
                self.output.warning(f"The minor of 1.2.3 is {v.minor}")
                self.output.warning(f"The patch of 1.2.3 is {v.patch}")
        """)
    c.save({"conanfile.py": conanfile})
    settings = "-s compiler=gcc -s compiler.libcxx=libstdc++11"
    c.run("create . {} -s compiler.version=11".format(settings))
    assert "The major of 1.2.3 is 1" in c.out
    assert "The minor of 1.2.3 is 2" in c.out
    assert "The patch of 1.2.3 is 3" in c.out

    c.run("create . {} -s compiler.version=9".format(settings), assert_error=True)



# -*- coding: utf-8 -*-
import os
import platform
import subprocess
import sys
import unittest
from collections import namedtuple

import pytest
import requests
import six
from bottle import request, static_file, HTTPError
from mock.mock import mock_open, patch
from parameterized import parameterized
from requests.models import Response

from conans.client import tools
from conans.client.cache.cache import CONAN_CONF
from conans.client.conan_api import ConanAPIV1
from conans.client.conf import get_default_client_conf
from conans.client.output import ConanOutput
from conans.client.tools import chdir
from conans.client.tools.files import replace_in_file
from conans.client.tools.oss import OSInfo
from conans.client.tools.win import vswhere
from conans.errors import ConanException, AuthenticationException
from conans.model.build_info import CppInfo
from conans.test.utils.mocks import ConanFileMock, TestBufferConanOutput
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import StoppableThreadBottle, TestClient, zipdir
from conans.tools import get_global_instances
from conans.util.files import load, md5, mkdir, save
from conans.util.runners import check_output_runner


class ConfigMock:
    def __init__(self):
        self.retry = 0
        self.retry_wait = 0


class RunnerMock(object):
    def __init__(self, return_ok=True, output=None):
        self.command_called = None
        self.return_ok = return_ok
        self.output = output

    def __call__(self, command, output, win_bash=False, subsystem=None):  # @UnusedVariable
        self.command_called = command
        self.win_bash = win_bash
        self.subsystem = subsystem
        if self.output and output and hasattr(output, "write"):
            output.write(self.output)
        return 0 if self.return_ok else 1


class ReplaceInFileTest(unittest.TestCase):
    def setUp(self):
        text = u'J\xe2nis\xa7'
        self.tmp_folder = temp_folder()

        self.win_file = os.path.join(self.tmp_folder, "win_encoding.txt")
        text = text.encode("Windows-1252", "ignore")
        with open(self.win_file, "wb") as handler:
            handler.write(text)

        self.bytes_file = os.path.join(self.tmp_folder, "bytes_encoding.txt")
        with open(self.bytes_file, "wb") as handler:
            handler.write(text)

    def test_replace_in_file(self):
        output = ConanOutput(sys.stdout)
        replace_in_file(self.win_file, "nis", "nus", output=output)
        replace_in_file(self.bytes_file, "nis", "nus", output=output)

        content = tools.load(self.win_file)
        self.assertNotIn("nis", content)
        self.assertIn("nus", content)

        content = tools.load(self.bytes_file)
        self.assertNotIn("nis", content)
        self.assertIn("nus", content)


class ToolsTest(unittest.TestCase):
    output = TestBufferConanOutput()

    def test_replace_paths(self):
        folder = temp_folder()
        path = os.path.join(folder, "file")
        replace_with = "MYPATH"
        expected = 'Some other contentsMYPATH"finally all text'

        out = TestBufferConanOutput()
        save(path, 'Some other contentsc:\\Path\\TO\\file.txt"finally all text')
        ret = tools.replace_path_in_file(path, "C:/Path/to/file.txt", replace_with,
                                         windows_paths=True, output=out)
        self.assertEqual(load(path), expected)
        self.assertTrue(ret)

        save(path, 'Some other contentsC:/Path\\TO\\file.txt"finally all text')
        ret = tools.replace_path_in_file(path, "C:/PATH/to/FILE.txt", replace_with,
                                         windows_paths=True, output=out)
        self.assertEqual(load(path), expected)
        self.assertTrue(ret)

        save(path, 'Some other contentsD:/Path\\TO\\file.txt"finally all text')
        ret = tools.replace_path_in_file(path, "C:/PATH/to/FILE.txt", replace_with, strict=False,
                                         windows_paths=True, output=out)
        self.assertEqual(load(path), 'Some other contentsD:/Path\\TO\\file.txt"finally all text')
        self.assertFalse(ret)

        # Multiple matches
        s = 'Some other contentsD:/Path\\TO\\file.txt"finally all textd:\\PATH\\to\\file.TXTMoretext'
        save(path, s)
        ret = tools.replace_path_in_file(path, "D:/PATH/to/FILE.txt", replace_with, strict=False,
                                         windows_paths=True, output=out)
        self.assertEqual(load(path), 'Some other contentsMYPATH"finally all textMYPATHMoretext')
        self.assertTrue(ret)

        # Automatic windows_paths
        save(path, s)
        ret = tools.replace_path_in_file(path, "D:/PATH/to/FILE.txt", replace_with, strict=False,
                                         output=out)
        if platform.system() == "Windows":
            self.assertEqual(load(path), 'Some other contentsMYPATH"finally all textMYPATHMoretext')
            self.assertTrue(ret)
        else:
            self.assertFalse(ret)

    def test_load_save(self):
        folder = temp_folder()
        path = os.path.join(folder, "file")
        save(path, u"äüïöñç")
        content = load(path)
        self.assertEqual(content, u"äüïöñç")

    def test_md5(self):
        result = md5(u"äüïöñç")
        self.assertEqual("dfcc3d74aa447280a7ecfdb98da55174", result)

    def test_cpu_count(self):
        output = ConanOutput(sys.stdout)
        cpus = tools.cpu_count(output=output)
        self.assertIsInstance(cpus, int)
        self.assertGreaterEqual(cpus, 1)
        with tools.environment_append({"CONAN_CPU_COUNT": "34"}):
            self.assertEqual(tools.cpu_count(output=output), 34)
        with tools.environment_append({"CONAN_CPU_COUNT": "null"}):
            with six.assertRaisesRegex(self, ConanException, "Invalid CONAN_CPU_COUNT value"):
                tools.cpu_count(output=output)

    @patch("conans.client.tools.oss.CpuProperties.get_cpu_period")
    @patch("conans.client.tools.oss.CpuProperties.get_cpu_quota")
    def test_cpu_count_in_container(self, get_cpu_quota_mock, get_cpu_period_mock):
        get_cpu_quota_mock.return_value = 12000
        get_cpu_period_mock.return_value = 1000

        output = ConanOutput(sys.stdout)
        cpus = tools.cpu_count(output=output)
        self.assertEqual(12, cpus)

    def test_get_env_unit(self):
        """
        Unit tests tools.get_env
        """
        # Test default
        self.assertIsNone(
            tools.get_env("NOT_DEFINED", environment={}),
            None
        )
        # Test defined default
        self.assertEqual(
            tools.get_env("NOT_DEFINED_KEY", default="random_default", environment={}),
            "random_default"
        )
        # Test return defined string
        self.assertEqual(
            tools.get_env("FROM_STR", default="", environment={"FROM_STR": "test_string_value"}),
            "test_string_value"
        )
        # Test boolean conversion
        self.assertEqual(
            tools.get_env("BOOL_FROM_STR", default=False, environment={"BOOL_FROM_STR": "1"}),
            True
        )
        self.assertEqual(
            tools.get_env("BOOL_FROM_STR", default=True, environment={"BOOL_FROM_STR": "0"}),
            False
        )
        self.assertEqual(
            tools.get_env("BOOL_FROM_STR", default=False, environment={"BOOL_FROM_STR": "True"}),
            True
        )
        self.assertEqual(
            tools.get_env("BOOL_FROM_STR", default=True, environment={"BOOL_FROM_STR": ""}),
            False
        )
        # Test int conversion
        self.assertEqual(
            tools.get_env("TO_INT", default=2, environment={"TO_INT": "1"}),
            1
        )
        # Test float conversion
        self.assertEqual(
            tools.get_env("TO_FLOAT", default=2.0, environment={"TO_FLOAT": "1"}),
            1.0
        ),
        # Test list conversion
        self.assertEqual(
            tools.get_env("TO_LIST", default=[], environment={"TO_LIST": "1,2,3"}),
            ["1", "2", "3"]
        )
        self.assertEqual(
            tools.get_env("TO_LIST_NOT_TRIMMED", default=[], environment={"TO_LIST_NOT_TRIMMED":
                                                                          " 1 , 2 , 3 "}),
            ["1", "2", "3"]
        )

    def test_get_env_in_conanfile(self):
        """
        Test get_env is available and working in conanfile
        """
        client = TestClient()

        conanfile = """from conans import ConanFile, tools

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"

    def build(self):
        run_tests = tools.get_env("CONAN_RUN_TESTS", default=False)
        print("test_get_env_in_conafile CONAN_RUN_TESTS=%r" % run_tests)
        assert(run_tests == True)
        """
        client.save({"conanfile.py": conanfile})

        with tools.environment_append({"CONAN_RUN_TESTS": "1"}):
            client.run("install .")
            client.run("build .")

    def test_global_tools_overrided(self):
        client = TestClient()

        conanfile = """
from conans import ConanFile, tools

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"

    def build(self):
        assert(tools._global_requester != None)
        assert(tools._global_output != None)
        """
        client.save({"conanfile.py": conanfile})

        client.run("install .")
        client.run("build .")

        # Not test the real commmand get_command if it's setting the module global vars
        tmp = temp_folder()
        conf = get_default_client_conf().replace("\n[proxies]", "\n[proxies]\nhttp = http://myproxy.com")
        os.mkdir(os.path.join(tmp, ".conan"))
        save(os.path.join(tmp, ".conan", CONAN_CONF), conf)
        with tools.environment_append({"CONAN_USER_HOME": tmp}):
            conan_api, _, _ = ConanAPIV1.factory()
        conan_api.remote_list()
        global_output, global_requester = get_global_instances()
        self.assertEqual(global_requester.proxies, {"http": "http://myproxy.com"})
        self.assertIsNotNone(global_output.warn)

    def test_environment_nested(self):
        with tools.environment_append({"A": "1", "Z": "40"}):
            with tools.environment_append({"A": "1", "B": "2"}):
                with tools.environment_append({"A": "2", "B": "2"}):
                    self.assertEqual(os.getenv("A"), "2")
                    self.assertEqual(os.getenv("B"), "2")
                    self.assertEqual(os.getenv("Z"), "40")
                self.assertEqual(os.getenv("A", None), "1")
                self.assertEqual(os.getenv("B", None), "2")
            self.assertEqual(os.getenv("A", None), "1")
            self.assertEqual(os.getenv("Z", None), "40")

        self.assertEqual(os.getenv("A", None), None)
        self.assertEqual(os.getenv("B", None), None)
        self.assertEqual(os.getenv("Z", None), None)

    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires vswhere")
    def test_vswhere_description_strip(self):
        myoutput = """
[
  {
    "instanceId": "17609d7c",
    "installDate": "2018-06-11T02:15:04Z",
    "installationName": "VisualStudio/15.7.3+27703.2026",
    "installationPath": "",
    "installationVersion": "15.7.27703.2026",
    "productId": "Microsoft.VisualStudio.Product.Enterprise",
    "productPath": "",
    "isPrerelease": false,
    "displayName": "Visual Studio Enterprise 2017",
    "description": "生産性向上と、さまざまな規模のチーム間の調整のための Microsoft DevOps ソリューション",
    "channelId": "VisualStudio.15.Release",
    "channelUri": "https://aka.ms/vs/15/release/channel",
    "enginePath": "",
    "releaseNotes": "https://go.microsoft.com/fwlink/?LinkId=660692#15.7.3",
    "thirdPartyNotices": "https://go.microsoft.com/fwlink/?LinkId=660708",
    "updateDate": "2018-06-11T02:15:04.7009868Z",
    "catalog": {
      "buildBranch": "d15.7",
      "buildVersion": "15.7.27703.2026",
      "id": "VisualStudio/15.7.3+27703.2026",
      "localBuild": "build-lab",
      "manifestName": "VisualStudio",
      "manifestType": "installer",
      "productDisplayVersion": "15.7.3",
      "productLine": "Dev15",
      "productLineVersion": "2017",
      "productMilestone": "RTW",
      "productMilestoneIsPreRelease": "False",
      "productName": "Visual Studio",
      "productPatchVersion": "3",
      "productPreReleaseMilestoneSuffix": "1.0",
      "productRelease": "RTW",
      "productSemanticVersion": "15.7.3+27703.2026",
      "requiredEngineVersion": "1.16.1187.57215"
    },
    "properties": {
      "campaignId": "",
      "canceled": "0",
      "channelManifestId": "VisualStudio.15.Release/15.7.3+27703.2026",
      "nickname": "",
      "setupEngineFilePath": ""
    }
  },
  {
    "instanceId": "VisualStudio.12.0",
    "installationPath": "",
    "installationVersion": "12.0"
  }
]

"""
        if six.PY3:
            # In python3 the output from subprocess.check_output are bytes, not str
            myoutput = myoutput.encode()
        myrunner = mock_open()
        myrunner.check_output = lambda x: myoutput
        with patch('conans.client.tools.win.subprocess', myrunner):
            json = vswhere()
            self.assertNotIn("descripton", json)

    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
    def test_run_in_bash(self):

        class MockConanfile(object):
            def __init__(self):

                self.output = namedtuple("output", "info")(lambda x: None)  # @UnusedVariable
                self.env = {"PATH": "/path/to/somewhere"}

                class MyRun(object):
                    def __call__(self, command, output, log_filepath=None,
                                 cwd=None, subprocess=False):  # @UnusedVariable
                        self.command = command
                self._conan_runner = MyRun()

        conanfile = MockConanfile()
        with patch.object(OSInfo, "bash_path", return_value='bash'):
            tools.run_in_windows_bash(conanfile, "a_command.bat", subsystem="cygwin")
            self.assertIn("bash", conanfile._conan_runner.command)
            self.assertIn("--login -c", conanfile._conan_runner.command)
            self.assertIn("^&^& a_command.bat ^", conanfile._conan_runner.command)

        with tools.environment_append({"CONAN_BASH_PATH": "path\\to\\mybash.exe"}):
            tools.run_in_windows_bash(conanfile, "a_command.bat", subsystem="cygwin")
            self.assertIn('path\\to\\mybash.exe --login -c', conanfile._conan_runner.command)

        with tools.environment_append({"CONAN_BASH_PATH": "path with spaces\\to\\mybash.exe"}):
            tools.run_in_windows_bash(conanfile, "a_command.bat", subsystem="cygwin",
                                      with_login=False)
            self.assertIn('"path with spaces\\to\\mybash.exe"  -c', conanfile._conan_runner.command)

        # try to append more env vars
        conanfile = MockConanfile()
        with patch.object(OSInfo, "bash_path", return_value='bash'):
            tools.run_in_windows_bash(conanfile, "a_command.bat", subsystem="cygwin",
                                      env={"PATH": "/other/path", "MYVAR": "34"})
            self.assertIn('^&^& PATH=\\^"/other/path:/path/to/somewhere:$PATH\\^" '
                          '^&^& MYVAR=34 ^&^& a_command.bat ^', conanfile._conan_runner.command)

    def test_download_retries_errors(self):
        out = TestBufferConanOutput()

        # Retry arguments override defaults
        with six.assertRaisesRegex(self, ConanException, "Error downloading"):
            tools.download("http://fakeurl3.es/nonexists",
                           os.path.join(temp_folder(), "file.txt"), out=out,
                           requester=requests,
                           retry=2, retry_wait=1)
        self.assertEqual(str(out).count("Waiting 1 seconds to retry..."), 2)

        # Not found error
        with six.assertRaisesRegex(self, ConanException,
                                   "Not found: http://google.es/FILE_NOT_FOUND"):
            tools.download("http://google.es/FILE_NOT_FOUND",
                           os.path.join(temp_folder(), "README.txt"), out=out,
                           requester=requests,
                           retry=2, retry_wait=0)

    @pytest.mark.slow
    def test_download_retries(self):
        http_server = StoppableThreadBottle()

        with tools.chdir(tools.mkdir_tmp()):
            with open("manual.html", "w") as fmanual:
                fmanual.write("this is some content")
                manual_file = os.path.abspath("manual.html")

        from bottle import auth_basic

        @http_server.server.get("/manual.html")
        def get_manual():
            return static_file(os.path.basename(manual_file),
                               os.path.dirname(manual_file))

        def check_auth(user, password):
            # Check user/password here
            return user == "user" and password == "passwd"

        @http_server.server.get('/basic-auth/<user>/<password>')
        @auth_basic(check_auth)
        def get_manual_auth(user, password):
            return static_file(os.path.basename(manual_file),
                               os.path.dirname(manual_file))

        http_server.run_server()

        out = TestBufferConanOutput()

        dest = os.path.join(temp_folder(), "manual.html")
        tools.download("http://localhost:%s/manual.html" % http_server.port, dest, out=out, retry=3,
                       retry_wait=0, requester=requests)
        self.assertTrue(os.path.exists(dest))
        content = load(dest)

        # overwrite = False
        with self.assertRaises(ConanException):
            tools.download("http://localhost:%s/manual.html" % http_server.port, dest, out=out,
                           retry=2, retry_wait=0, overwrite=False, requester=requests)

        # overwrite = True
        tools.download("http://localhost:%s/manual.html" % http_server.port, dest, out=out, retry=2,
                       retry_wait=0, overwrite=True, requester=requests)
        self.assertTrue(os.path.exists(dest))
        content_new = load(dest)
        self.assertEqual(content, content_new)

        # Not authorized
        with self.assertRaises(ConanException):
            tools.download("http://localhost:%s/basic-auth/user/passwd" % http_server.port, dest,
                           overwrite=True, requester=requests, out=out, retry=0, retry_wait=0)

        # Authorized
        tools.download("http://localhost:%s/basic-auth/user/passwd" % http_server.port, dest,
                       auth=("user", "passwd"), overwrite=True, requester=requests, out=out,
                       retry=0, retry_wait=0)

        # Authorized using headers
        tools.download("http://localhost:%s/basic-auth/user/passwd" % http_server.port, dest,
                       headers={"Authorization": "Basic dXNlcjpwYXNzd2Q="}, overwrite=True,
                       requester=requests, out=out, retry=0, retry_wait=0)
        http_server.stop()

    @pytest.mark.slow
    @patch("conans.tools._global_config")
    def test_download_unathorized(self, mock_config):
        http_server = StoppableThreadBottle()
        mock_config.return_value = ConfigMock()

        @http_server.server.get('/forbidden')
        def get_forbidden():
            return HTTPError(403, "Access denied.")

        http_server.run_server()

        out = TestBufferConanOutput()
        dest = os.path.join(temp_folder(), "manual.html")
        # Not authorized
        with six.assertRaisesRegex(self, AuthenticationException, "403"):
            tools.download("http://localhost:%s/forbidden" % http_server.port, dest,
                           requester=requests, out=out)

        http_server.stop()

    @parameterized.expand([
        ["Linux", "x86", None, "x86-linux-gnu"],
        ["Linux", "x86_64", None, "x86_64-linux-gnu"],
        ["Linux", "armv6", None, "arm-linux-gnueabi"],
        ["Linux", "sparc", None, "sparc-linux-gnu"],
        ["Linux", "sparcv9", None, "sparc64-linux-gnu"],
        ["Linux", "mips", None, "mips-linux-gnu"],
        ["Linux", "mips64", None, "mips64-linux-gnu"],
        ["Linux", "ppc32", None, "powerpc-linux-gnu"],
        ["Linux", "ppc64", None, "powerpc64-linux-gnu"],
        ["Linux", "ppc64le", None, "powerpc64le-linux-gnu"],
        ["Linux", "armv5te", None, "arm-linux-gnueabi"],
        ["Linux", "arm_whatever", None, "arm-linux-gnueabi"],
        ["Linux", "armv7hf", None, "arm-linux-gnueabihf"],
        ["Linux", "armv6", None, "arm-linux-gnueabi"],
        ["Linux", "armv7", None, "arm-linux-gnueabi"],
        ["Linux", "armv8_32", None, "aarch64-linux-gnu_ilp32"],
        ["Linux", "armv5el", None, "arm-linux-gnueabi"],
        ["Linux", "armv5hf", None, "arm-linux-gnueabihf"],
        ["Linux", "s390", None, "s390-ibm-linux-gnu"],
        ["Linux", "s390x", None, "s390x-ibm-linux-gnu"],
        ["Android", "x86", None, "i686-linux-android"],
        ["Android", "x86_64", None, "x86_64-linux-android"],
        ["Android", "armv6", None, "arm-linux-androideabi"],
        ["Android", "armv7", None, "arm-linux-androideabi"],
        ["Android", "armv7hf", None, "arm-linux-androideabi"],
        ["Android", "armv8", None, "aarch64-linux-android"],
        ["Windows", "x86", "Visual Studio", "i686-windows-msvc"],
        ["Windows", "x86", "gcc", "i686-w64-mingw32"],
        ["Windows", "x86_64", "gcc", "x86_64-w64-mingw32"],
        ["Darwin", "x86_64", None, "x86_64-apple-darwin"],
        ["Macos", "x86", None, "i686-apple-darwin"],
        ["iOS", "armv7", None, "arm-apple-ios"],
        ["iOS", "x86", None, "i686-apple-ios"],
        ["iOS", "x86_64", None, "x86_64-apple-ios"],
        ["watchOS", "armv7k", None, "arm-apple-watchos"],
        ["watchOS", "armv8_32", None, "aarch64-apple-watchos"],
        ["watchOS", "x86", None, "i686-apple-watchos"],
        ["watchOS", "x86_64", None, "x86_64-apple-watchos"],
        ["tvOS", "armv8", None, "aarch64-apple-tvos"],
        ["tvOS", "armv8.3", None, "aarch64-apple-tvos"],
        ["tvOS", "x86", None, "i686-apple-tvos"],
        ["tvOS", "x86_64", None, "x86_64-apple-tvos"],
        ["Emscripten", "asm.js", None, "asmjs-local-emscripten"],
        ["Emscripten", "wasm", None, "wasm32-local-emscripten"],
        ["AIX", "ppc32", None, "rs6000-ibm-aix"],
        ["AIX", "ppc64", None, "powerpc-ibm-aix"],
        ["Neutrino", "armv7", None, "arm-nto-qnx"],
        ["Neutrino", "armv8", None, "aarch64-nto-qnx"],
        ["Neutrino", "sh4le", None, "sh4-nto-qnx"],
        ["Neutrino", "ppc32be", None, "powerpcbe-nto-qnx"],
        ["Linux", "e2k-v2", None, "e2k-unknown-linux-gnu"],
        ["Linux", "e2k-v3", None, "e2k-unknown-linux-gnu"],
        ["Linux", "e2k-v4", None, "e2k-unknown-linux-gnu"],
        ["Linux", "e2k-v5", None, "e2k-unknown-linux-gnu"],
        ["Linux", "e2k-v6", None, "e2k-unknown-linux-gnu"],
        ["Linux", "e2k-v7", None, "e2k-unknown-linux-gnu"],
    ])
    def test_get_gnu_triplet(self, os, arch, compiler, expected_triplet):
        triplet = tools.get_gnu_triplet(os, arch, compiler)
        self.assertEqual(triplet, expected_triplet,
                         "triplet did not match for ('%s', '%s', '%s')" % (os, arch, compiler))

    def test_get_gnu_triplet_on_windows_without_compiler(self):
        with self.assertRaises(ConanException):
            tools.get_gnu_triplet("Windows", "x86")

    def test_detect_windows_subsystem(self):
        # Don't raise test
        result = OSInfo.detect_windows_subsystem()
        if not OSInfo.bash_path() or platform.system() != "Windows":
            self.assertEqual(None, result)
        else:
            self.assertEqual(str, type(result))

    @pytest.mark.slow
    @pytest.mark.local_bottle
    def test_get_filename_download(self):
        # Create a tar file to be downloaded from server
        with tools.chdir(tools.mkdir_tmp()):
            import tarfile
            tar_file = tarfile.open("sample.tar.gz", "w:gz")
            mkdir("test_folder")
            tar_file.add(os.path.abspath("test_folder"), "test_folder")
            tar_file.close()
            file_path = os.path.abspath("sample.tar.gz")
            assert(os.path.exists(file_path))

        # Instance stoppable thread server and add endpoints
        thread = StoppableThreadBottle()

        @thread.server.get("/this_is_not_the_file_name")
        def get_file():
            return static_file(os.path.basename(file_path), root=os.path.dirname(file_path))

        @thread.server.get("/")
        def get_file2():
            self.assertEqual(request.query["file"], "1")
            return static_file(os.path.basename(file_path), root=os.path.dirname(file_path))

        @thread.server.get("/error_url")
        def error_url():
            from bottle import response
            response.status = 500
            return 'This always fail'

        thread.run_server()

        out = TestBufferConanOutput()
        # Test: File name cannot be deduced from '?file=1'
        with self.assertRaises(ConanException) as error:
            tools.get("http://localhost:%s/?file=1" % thread.port, output=out)
        self.assertIn("Cannot deduce file name from the url: 'http://localhost:{}/?file=1'."
                      " Use 'filename' parameter.".format(thread.port), str(error.exception))

        # Test: Works with filename parameter instead of '?file=1'
        with tools.chdir(tools.mkdir_tmp()):
            tools.get("http://localhost:%s/?file=1" % thread.port, filename="sample.tar.gz",
                      requester=requests, output=out, retry=0, retry_wait=0)
            self.assertTrue(os.path.exists("test_folder"))

        # Test: Use a different endpoint but still not the filename one
        with tools.chdir(tools.mkdir_tmp()):
            from zipfile import BadZipfile
            with self.assertRaises(BadZipfile):
                tools.get("http://localhost:%s/this_is_not_the_file_name" % thread.port,
                          requester=requests, output=out, retry=0, retry_wait=0)
            tools.get("http://localhost:%s/this_is_not_the_file_name" % thread.port,
                      filename="sample.tar.gz", requester=requests, output=out,
                      retry=0, retry_wait=0)
            self.assertTrue(os.path.exists("test_folder"))
        thread.stop()

        with six.assertRaisesRegex(self, ConanException, "Error"):
            tools.get("http://localhost:%s/error_url" % thread.port,
                      filename="fake_sample.tar.gz", requester=requests, output=out, verify=False,
                      retry=2, retry_wait=0)

        # Not found error
        self.assertEqual(str(out).count("Waiting 0 seconds to retry..."), 2)

    def test_get_unzip_strip_root(self):
        """Test that the strip_root mechanism from the underlying unzip
          is called if I call the tools.get by checking that the exception of an invalid zip to
          flat is raised"""

        zip_folder = temp_folder()

        def mock_download(*args, **kwargs):
            tmp_folder = temp_folder()
            with chdir(tmp_folder):
                ori_files_dir = os.path.join(tmp_folder, "subfolder-1.2.3")
                file1 = os.path.join(ori_files_dir, "file1")
                file2 = os.path.join(ori_files_dir, "folder", "file2")
                # !!! This file is not under the root "subfolder-1.2.3"
                file3 = os.path.join("file3")
                save(file1, "")
                save(file2, "")
                save(file3, "")
                zip_file = os.path.join(zip_folder, "file.zip")
                zipdir(tmp_folder, zip_file)

        with six.assertRaisesRegex(self, ConanException, "The zip file contains more than 1 "
                                                         "folder in the root"):
            with patch('conans.client.tools.net.download', new=mock_download):
                with chdir(zip_folder):
                    tools.get("file.zip", strip_root=True)

    @pytest.mark.slow
    @pytest.mark.local_bottle
    def test_get_gunzip(self):
        # Create a tar file to be downloaded from server
        tmp = temp_folder()
        filepath = os.path.join(tmp, "test.txt.gz")
        import gzip
        with gzip.open(filepath, "wb") as f:
            f.write(b"hello world zipped!")

        thread = StoppableThreadBottle()

        @thread.server.get("/test.txt.gz")
        def get_file():
            return static_file(os.path.basename(filepath), root=os.path.dirname(filepath),
                               mimetype="application/octet-stream")

        thread.run_server()

        out = TestBufferConanOutput()
        with tools.chdir(tools.mkdir_tmp()):
            tools.get("http://localhost:%s/test.txt.gz" % thread.port, requester=requests,
                      output=out, retry=0, retry_wait=0)
            self.assertTrue(os.path.exists("test.txt"))
            self.assertEqual(load("test.txt"), "hello world zipped!")
        with tools.chdir(tools.mkdir_tmp()):
            tools.get("http://localhost:%s/test.txt.gz" % thread.port, requester=requests,
                      output=out, destination="myfile.doc", retry=0, retry_wait=0)
            self.assertTrue(os.path.exists("myfile.doc"))
            self.assertEqual(load("myfile.doc"), "hello world zipped!")
        with tools.chdir(tools.mkdir_tmp()):
            tools.get("http://localhost:%s/test.txt.gz" % thread.port, requester=requests,
                      output=out, destination="mytemp/myfile.txt", retry=0, retry_wait=0)
            self.assertTrue(os.path.exists("mytemp/myfile.txt"))
            self.assertEqual(load("mytemp/myfile.txt"), "hello world zipped!")

        thread.stop()

    @patch("conans.client.tools.net.unzip")
    def test_get_mirror(self, _):
        """ tools.get must supports a list of URLs. However, only one must be downloaded.
        """

        class MockRequester(object):
            def __init__(self):
                self.count = 0
                self.fail_first = False
                self.fail_all = False

            def get(self, *args, **kwargs):
                self.count += 1
                resp = Response()
                resp._content = b'{"results": []}'
                resp.headers = {"Content-Type": "application/json"}
                resp.status_code = 200
                if (self.fail_first and self.count == 1) or self.fail_all:
                    resp.status_code = 408
                return resp

        file = "test.txt.gz"
        out = TestBufferConanOutput()
        urls = ["http://localhost:{}/{}".format(8000 + i, file) for i in range(3)]

        # Only the first file must be downloaded
        with tools.chdir(tools.mkdir_tmp()):
            requester = MockRequester()
            tools.get(urls, requester=requester, output=out, retry=0, retry_wait=0)
            self.assertEqual(1, requester.count)

        # Fail the first, download only the second
        with tools.chdir(tools.mkdir_tmp()):
            requester = MockRequester()
            requester.fail_first = True
            tools.get(urls, requester=requester, output=out, retry=0, retry_wait=0)
            self.assertEqual(2, requester.count)
            self.assertIn("WARN: Could not download from the URL {}: Error 408 downloading file {}."
                          " Trying another mirror."
                          .format(urls[0], urls[0]), out)

        # Fail all downloads
        with tools.chdir(tools.mkdir_tmp()):
            requester = MockRequester()
            requester.fail_all = True
            with self.assertRaises(ConanException) as error:
                tools.get(urls, requester=requester, output=out, retry=0, retry_wait=0)
            self.assertEqual(3, requester.count)
            self.assertIn("All downloads from (3) URLs have failed.", str(error.exception))

    def test_check_output_runner(self):
        original_temp = temp_folder()
        patched_temp = os.path.join(original_temp, "dir with spaces")
        payload = "hello world"
        output = check_output_runner(["echo", payload], stderr=subprocess.STDOUT)
        self.assertIn(payload, str(output))

    def test_unix_to_dos_conanfile(self):
        client = TestClient()
        conanfile = """
import os
from conans import ConanFile, tools

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    exports_sources = "file.txt"

    def build(self):
        assert("\\r\\n" in tools.load("file.txt"))
        tools.dos2unix("file.txt")
        assert("\\r\\n" not in tools.load("file.txt"))
        tools.unix2dos("file.txt")
        assert("\\r\\n" in tools.load("file.txt"))
"""
        client.save({"conanfile.py": conanfile, "file.txt": "hello\r\n"})
        client.run("create . user/channel")


class CollectLibTestCase(unittest.TestCase):

    def test_collect_libs(self):
        conanfile = ConanFileMock()
        # Without package_folder
        result = tools.collect_libs(conanfile)
        self.assertEqual([], result)

        # Default behavior
        conanfile.folders.set_base_package(temp_folder())
        mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
        save(mylib_path, "")
        conanfile.cpp_info = CppInfo("", "")
        result = tools.collect_libs(conanfile)
        self.assertEqual(["mylib"], result)

        # Custom folder
        customlib_path = os.path.join(conanfile.package_folder, "custom_folder", "customlib.lib")
        save(customlib_path, "")
        result = tools.collect_libs(conanfile, folder="custom_folder")
        self.assertEqual(["customlib"], result)

        # Custom folder doesn't exist
        result = tools.collect_libs(conanfile, folder="fake_folder")
        self.assertEqual([], result)
        self.assertIn("Lib folder doesn't exist, can't collect libraries:", conanfile.output)

        # Use cpp_info.libdirs
        conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
        result = tools.collect_libs(conanfile)
        self.assertEqual(["customlib", "mylib"], result)

        # Custom folder with multiple libdirs should only collect from custom folder
        self.assertEqual(["lib", "custom_folder"], conanfile.cpp_info.libdirs)
        result = tools.collect_libs(conanfile, folder="custom_folder")
        self.assertEqual(["customlib"], result)

        # Unicity of lib names
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(temp_folder())
        conanfile.cpp_info = CppInfo(conanfile.name, "")
        custom_mylib_path = os.path.join(conanfile.package_folder, "custom_folder", "mylib.lib")
        lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
        save(custom_mylib_path, "")
        save(lib_mylib_path, "")
        conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
        result = tools.collect_libs(conanfile)
        self.assertEqual(["mylib"], result)

        # Warn lib folder does not exist with correct result
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(temp_folder())
        conanfile.cpp_info = CppInfo(conanfile.name, "")
        lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
        save(lib_mylib_path, "")
        no_folder_path = os.path.join(conanfile.package_folder, "no_folder")
        conanfile.cpp_info.libdirs = ["no_folder", "lib"]  # 'no_folder' does NOT exist
        result = tools.collect_libs(conanfile)
        self.assertEqual(["mylib"], result)
        self.assertIn("WARN: Lib folder doesn't exist, can't collect libraries: %s"
                      % no_folder_path, conanfile.output)

    @pytest.mark.skipif(platform.system() == "Windows", reason="Needs symlinks support")
    def test_collect_libs_symlinks(self):
        # Keep only the shortest lib name per group of symlinks
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(temp_folder())
        conanfile.cpp_info = CppInfo(conanfile.name, "")
        version_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.0.0.dylib")
        soversion_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.dylib")
        lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.dylib")
        lib_mylib2_path = os.path.join(conanfile.package_folder, "lib", "libmylib.2.dylib")
        lib_mylib3_path = os.path.join(conanfile.package_folder, "custom_folder", "libmylib.3.dylib")
        save(version_mylib_path, "")
        os.symlink(version_mylib_path, soversion_mylib_path)
        os.symlink(soversion_mylib_path, lib_mylib_path)
        save(lib_mylib2_path, "")
        save(lib_mylib3_path, "")
        conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
        result = tools.collect_libs(conanfile)
        self.assertEqual(["mylib", "mylib.2", "mylib.3"], result)

    def test_self_collect_libs(self):
        conanfile = ConanFileMock()
        # Without package_folder
        result = conanfile.collect_libs()
        self.assertEqual([], result)

        # Default behavior
        conanfile.folders.set_base_package(temp_folder())
        mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
        save(mylib_path, "")
        conanfile.cpp_info = CppInfo("", "")
        result = conanfile.collect_libs()
        self.assertEqual(["mylib"], result)

        # Custom folder
        customlib_path = os.path.join(conanfile.package_folder, "custom_folder", "customlib.lib")
        save(customlib_path, "")
        result = conanfile.collect_libs(folder="custom_folder")
        self.assertEqual(["customlib"], result)

        # Custom folder doesn't exist
        result = conanfile.collect_libs(folder="fake_folder")
        self.assertEqual([], result)
        self.assertIn("Lib folder doesn't exist, can't collect libraries:", conanfile.output)

        # Use cpp_info.libdirs
        conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
        result = conanfile.collect_libs()
        self.assertEqual(["customlib", "mylib"], result)

        # Custom folder with multiple libdirs should only collect from custom folder
        self.assertEqual(["lib", "custom_folder"], conanfile.cpp_info.libdirs)
        result = conanfile.collect_libs(folder="custom_folder")
        self.assertEqual(["customlib"], result)

        # Unicity of lib names
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(temp_folder())
        conanfile.cpp_info = CppInfo("", "")
        custom_mylib_path = os.path.join(conanfile.package_folder, "custom_folder", "mylib.lib")
        lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
        save(custom_mylib_path, "")
        save(lib_mylib_path, "")
        conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
        result = conanfile.collect_libs()
        self.assertEqual(["mylib"], result)

        # Warn lib folder does not exist with correct result
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(temp_folder())
        conanfile.cpp_info = CppInfo("", "")
        lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
        save(lib_mylib_path, "")
        no_folder_path = os.path.join(conanfile.package_folder, "no_folder")
        conanfile.cpp_info.libdirs = ["no_folder", "lib"]  # 'no_folder' does NOT exist
        result = conanfile.collect_libs()
        self.assertEqual(["mylib"], result)
        self.assertIn("WARN: Lib folder doesn't exist, can't collect libraries: %s"
                      % no_folder_path, conanfile.output)

    @pytest.mark.skipif(platform.system() == "Windows", reason="Needs symlinks support")
    def test_self_collect_libs_symlinks(self):
        # Keep only the shortest lib name per group of symlinks
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(temp_folder())
        conanfile.cpp_info = CppInfo("", "")
        version_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.0.0.dylib")
        soversion_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.dylib")
        lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.dylib")
        lib_mylib2_path = os.path.join(conanfile.package_folder, "lib", "libmylib.2.dylib")
        lib_mylib3_path = os.path.join(conanfile.package_folder, "custom_folder", "libmylib.3.dylib")
        save(version_mylib_path, "")
        os.symlink(version_mylib_path, soversion_mylib_path)
        os.symlink(soversion_mylib_path, lib_mylib_path)
        save(lib_mylib2_path, "")
        save(lib_mylib3_path, "")
        conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
        result = conanfile.collect_libs()
        self.assertEqual(["mylib", "mylib.2", "mylib.3"], result)



import unittest

import pytest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient, TestServer, TurboTestClient, GenConanfile
from conans.util.env_reader import get_env


class DownloadRevisionsTest(unittest.TestCase):

    @pytest.mark.skipif(get_env("TESTING_REVISIONS_ENABLED", False), reason="No sense with revs")
    def test_download_revs_disabled_with_rrev(self):
        # https://github.com/conan-io/conan/issues/6106
        client = TestClient(revisions_enabled=False)
        client.run("download pkg/1.0@user/channel#fakerevision", assert_error=True)
        self.assertIn(
            "ERROR: Revisions not enabled in the client, specify a reference without revision",
            client.out)

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_download_revs_enabled_with_fake_rrev(self):
        client = TestClient(default_server_user=True, revisions_enabled=True)
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . pkg/1.0@user/channel")
        client.run("upload * --all --confirm")
        client.run("remove * -f")
        client.run("download pkg/1.0@user/channel#fakerevision", assert_error=True)
        self.assertIn("ERROR: Recipe not found: 'pkg/1.0@user/channel'", client.out)

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_download_revs_enabled_with_rrev(self):
        ref = ConanFileReference.loads("pkg/1.0@user/channel")
        client = TurboTestClient(default_server_user=True, revisions_enabled=True)
        pref = client.create(ref, conanfile=GenConanfile())
        client.run("upload pkg/1.0@user/channel --all --confirm")
        # create new revision from recipe
        client.create(ref, conanfile=GenConanfile().with_build_msg("new revision"))
        client.run("upload pkg/1.0@user/channel --all --confirm")
        client.run("remove * -f")
        client.run("download pkg/1.0@user/channel#{}".format(pref.ref.revision))
        self.assertIn("pkg/1.0@user/channel: Package installed {}".format(pref.id), client.out)
        search_result = client.search("pkg/1.0@user/channel --revisions")[0]
        self.assertIn(pref.ref.revision, search_result["revision"])

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_download_revs_enabled_with_rrev_no_user_channel(self):
        ref = ConanFileReference.loads("pkg/1.0@")
        servers = {"default": TestServer([("*/*@*/*", "*")], [("*/*@*/*", "*")],
                                         users={"user": "password"})}
        client = TurboTestClient(servers=servers, revisions_enabled=True,
                                 users={"default": [("user", "password")]})
        pref = client.create(ref, conanfile=GenConanfile())
        client.run("upload pkg/1.0@ --all --confirm")
        # create new revision from recipe
        client.create(ref, conanfile=GenConanfile().with_build_msg("new revision"))
        client.run("upload pkg/1.0@ --all --confirm")
        client.run("remove * -f")
        client.run("download pkg/1.0@#{}".format(pref.ref.revision))
        self.assertIn("pkg/1.0: Package installed {}".format(pref.id), client.out)
        search_result = client.search("pkg/1.0@ --revisions")[0]
        self.assertIn(pref.ref.revision, search_result["revision"])

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_download_revs_enabled_with_prev(self):
        # https://github.com/conan-io/conan/issues/6106
        ref = ConanFileReference.loads("pkg/1.0@user/channel")
        client = TurboTestClient(default_server_user=True, revisions_enabled=True)
        pref = client.create(ref, conanfile=GenConanfile())
        client.run("upload pkg/1.0@user/channel --all --confirm")
        client.create(ref, conanfile=GenConanfile().with_build_msg("new revision"))
        client.run("upload pkg/1.0@user/channel --all --confirm")
        client.run("remove * -f")
        client.run("download pkg/1.0@user/channel#{}:{}#{}".format(pref.ref.revision,
                                                                   pref.id,
                                                                   pref.revision))
        self.assertIn("pkg/1.0@user/channel: Package installed {}".format(pref.id), client.out)
        search_result = client.search("pkg/1.0@user/channel --revisions")[0]
        self.assertIn(pref.ref.revision, search_result["revision"])
        search_result = client.search(
            "pkg/1.0@user/channel#{}:{} --revisions".format(pref.ref.revision, pref.id))[0]
        self.assertIn(pref.revision, search_result["revision"])



import platform
import textwrap

import pytest
from parameterized import parameterized

from conans.test.utils.tools import TestClient


@parameterized.expand([("msvc", "190", "dynamic"),
                       ("msvc", "191", "static")]
                      )
@pytest.mark.tool_visual_studio
@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
def test_toolchain_win(compiler, version, runtime):
    client = TestClient(path_with_spaces=False)
    settings = {"compiler": compiler,
                "compiler.version": version,
                "compiler.cppstd": "17",
                "compiler.runtime": runtime,
                "build_type": "Release",
                "arch": "x86_64"}

    # Build the profile according to the settings provided
    settings = " ".join('-s %s="%s"' % (k, v) for k, v in settings.items() if v)

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.microsoft import MSBuildToolchain
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def generate(self):
                msbuild = MSBuildToolchain(self)
                msbuild.generate()
            """)
    client.save({"conanfile.py": conanfile})
    client.run("install . {}".format(settings))
    props = client.load("conantoolchain_release_x64.props")
    assert "<LanguageStandard>stdcpp17</LanguageStandard>" in props
    if version == "190":
        assert "<PlatformToolset>v140</PlatformToolset>" in props
    else:
        assert "<PlatformToolset>v141</PlatformToolset>" in props
    if runtime == "dynamic":
        assert "<RuntimeLibrary>MultiThreadedDLL</RuntimeLibrary>" in props
    else:
        assert "<RuntimeLibrary>MultiThreaded</RuntimeLibrary>" in props



import os
import textwrap
import unittest

from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient


class UserInfoTest(unittest.TestCase):

    def test_user_info_propagation(self):
        client = TestClient()

        def export_lib(name, requires, infolines):
            base = textwrap.dedent("""
                from conans import ConanFile

                class MyConanfile(ConanFile):
                    name = "%s"
                    version = "0.1"
                    requires = "%s"

                    def package_info(self):
                        %s
                    """)
            client.save({CONANFILE: base % (name, requires, infolines)}, clean_first=True)
            client.run("export . lasote/stable")

        export_lib("LIB_A", "", "self.user_info.VAR1=2")
        export_lib("LIB_B", "LIB_A/0.1@lasote/stable", "self.user_info.VAR1=2\n        "
                                                       "self.user_info.VAR2=3")
        export_lib("LIB_C", "LIB_B/0.1@lasote/stable", "self.user_info.VAR1=2")
        export_lib("LIB_D", "LIB_C/0.1@lasote/stable", "self.user_info.var1=2")

        reuse = textwrap.dedent("""
            from conans import ConanFile

            class MyConanfile(ConanFile):
                requires = "LIB_D/0.1@lasote/stable"

                def build(self):
                    assert(self.deps_user_info["LIB_A"].VAR1=="2")
                    assert(self.deps_user_info["LIB_B"].VAR1=="2")
                    assert(self.deps_user_info["LIB_B"].VAR2=="3")
                    assert(self.deps_user_info["LIB_C"].VAR1=="2")
                    assert(self.deps_user_info["LIB_D"].var1=="2")
                    # Idiomatic way to check for attribute existence
                    # https://github.com/conan-io/conan/issues/7130
                    assert(hasattr(self.deps_user_info["LIB_A"], "VAR1"))
                    assert(not hasattr(self.deps_user_info["LIB_A"], "NONEXIST"))
                    assert(getattr(self.deps_user_info["LIB_A"], "VAR1"))
                    assert(not getattr(self.deps_user_info["LIB_A"], "NONEXIST", None))
                """)
        client.save({CONANFILE: reuse}, clean_first=True)
        client.run("export . reuse/0.1@lasote/stable")
        client.run('install reuse/0.1@lasote/stable --build -g txt')

        # Assert generator TXT
        txt_contents = client.load("conanbuildinfo.txt")
        self.assertIn("[USER_LIB_A]%sVAR1=2" % os.linesep, txt_contents)
        self.assertIn("[USER_LIB_B]%sVAR1=2%sVAR2=3" % (os.linesep, os.linesep), txt_contents)
        self.assertIn("[USER_LIB_C]%sVAR1=2" % os.linesep, txt_contents)
        self.assertIn("[USER_LIB_D]%svar1=2" % os.linesep, txt_contents)

        # Now try local command with a consumer
        client.run('install . --build')
        client.run("build .")



import textwrap

from mock import patch
from requests import ConnectionError

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient, NO_SETTINGS_PACKAGE_ID, TestRequester


def test_upload_parallel_error():
    """Cause an error in the parallel transfer and see some message"""

    class FailOnReferencesUploader(TestRequester):
        fail_on = ["lib1", "lib3"]

        def put(self, *args, **kwargs):
            if any(ref in args[0] for ref in self.fail_on):
                raise ConnectionError("Connection fails with lib2 and lib4 references!")
            else:
                return super(FailOnReferencesUploader, self).put(*args, **kwargs)

    client = TestClient(requester_class=FailOnReferencesUploader, default_server_user=True)
    client.save({"conanfile.py": GenConanfile()})
    client.run('user -p password -r default user')
    for index in range(4):
        client.run('create . lib{}/1.0@user/channel'.format(index))
    client.run('upload lib* --parallel -c --all -r default --retry-wait=0', assert_error=True)
    assert "Connection fails with lib2 and lib4 references!" in client.out
    assert "Execute upload again to retry upload the failed files" in client.out


def test_upload_parallel_success():
    """Upload 2 packages in parallel with success"""

    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": GenConanfile()})
    client.run('create . lib0/1.0@user/channel')
    assert "lib0/1.0@user/channel: Package '{}' created".format(NO_SETTINGS_PACKAGE_ID) in client.out
    client.run('create . lib1/1.0@user/channel')
    assert "lib1/1.0@user/channel: Package '{}' created".format(NO_SETTINGS_PACKAGE_ID) in client.out
    client.run('user -p password -r default user')
    client.run('upload lib* --parallel -c --all -r default')
    assert "Uploading lib0/1.0@user/channel to remote 'default'" in client.out
    assert "Uploading lib1/1.0@user/channel to remote 'default'" in client.out
    client.run('search lib0/1.0@user/channel -r default')
    assert "lib0/1.0@user/channel" in client.out
    client.run('search lib1/1.0@user/channel -r default')
    assert "lib1/1.0@user/channel" in client.out


def test_upload_parallel_fail_on_interaction():
    """Upload 2 packages in parallel and fail because non_interactive forced"""

    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": GenConanfile()})
    num_references = 2
    for index in range(num_references):
        client.run('create . lib{}/1.0@user/channel'.format(index))
        assert "lib{}/1.0@user/channel: Package '{}' created".format(
            index,
            NO_SETTINGS_PACKAGE_ID) in client.out
    client.run('user -c')
    client.run('upload lib* --parallel -c --all -r default', assert_error=True)
    assert "ERROR: lib0/1.0@user/channel: Upload recipe to 'default' failed: " \
           "Conan interactive mode disabled. [Remote: default]" in client.out


def test_beat_character_long_upload():
    client = TestClient(default_server_user=True)
    slow_conanfile = textwrap.dedent("""
        from conans import ConanFile
        class MyPkg(ConanFile):
            exports = "*"
            def package(self):
                self.copy("*")
        """)
    client.save({"conanfile.py": slow_conanfile,
                 "hello.cpp": ""})
    client.run("create . pkg/0.1@user/stable")
    client.run("user user --password=password")
    with patch("conans.util.progress_bar.TIMEOUT_BEAT_SECONDS", -1):
        with patch("conans.util.progress_bar.TIMEOUT_BEAT_CHARACTER", "%&$"):
            client.run("upload pkg/0.1@user/stable --all")
    out = "".join(str(client.out).splitlines())
    assert "Compressing package...%&$%&$Uploading conan_package.tgz -> pkg/0.1@user/stable:5ab8" \
           in out
    assert "%&$Uploading conan_export.tgz" in out
    assert "%&$Uploading conaninfo.txt" in out



import os
import textwrap

import pytest

from conan.tools.cmake.cmakedeps.cmakedeps import FIND_MODE_CONFIG, FIND_MODE_MODULE, FIND_MODE_BOTH, \
    FIND_MODE_NONE
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


@pytest.mark.parametrize("cmake_find_mode", [FIND_MODE_CONFIG, FIND_MODE_MODULE,
                                             FIND_MODE_BOTH, FIND_MODE_NONE, None])
def test_reuse_with_modules_and_config(cmake_find_mode):
    t = TestClient()
    conanfile = textwrap.dedent("""
            import os
            from conans import ConanFile

            class Conan(ConanFile):
                name = "mydep"
                version = "1.0"
                settings = "os", "arch", "compiler", "build_type"

                def package_info(self):
                    {}

            """)

    if cmake_find_mode is not None:
        s = 'self.cpp_info.set_property("cmake_find_mode", "{}")'.format(cmake_find_mode)
        conanfile = conanfile.format(s)
    t.save({"conanfile.py": conanfile})
    t.run("create .")

    conanfile = GenConanfile().with_name("myapp").with_require("mydep/1.0")\
                                                 .with_generator("CMakeDeps")\
                                                 .with_settings("build_type", "os", "arch", "compiler")
    t.save({"conanfile.py": conanfile})

    t.run("install . -if=install")

    ifolder = os.path.join(t.current_folder, "install")

    def exists_config(ifolder):
        return os.path.exists(os.path.join(ifolder, "mydep-config.cmake"))

    def exists_module(ifolder):
        return os.path.exists(os.path.join(ifolder, "Findmydep.cmake"))

    if cmake_find_mode == FIND_MODE_CONFIG or cmake_find_mode is None:
        # None is default "config"
        assert exists_config(ifolder)
        assert not exists_module(ifolder)
    elif cmake_find_mode == FIND_MODE_MODULE:
        assert not exists_config(ifolder)
        assert exists_module(ifolder)
    elif cmake_find_mode == FIND_MODE_BOTH:
        assert exists_config(ifolder)
        assert exists_module(ifolder)
    elif cmake_find_mode == FIND_MODE_NONE:
        assert not exists_config(ifolder)
        assert not exists_module(ifolder)



import os
import unittest

from conans.model.ref import ConanFileReference
from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.files import load


class HalfDiamondTest(unittest.TestCase):

    def setUp(self):
        self.client = TestClient()

    def _export(self, name, deps=None, export=True):

        conanfile = GenConanfile().with_name(name).with_version("0.1")\
                                  .with_option("potato", [True, False])\
                                  .with_default_option("potato", True)
        if deps:
            for dep in deps:
                ref = ConanFileReference.loads(dep)
                conanfile = conanfile.with_require(ref)

        conanfile = str(conanfile) + """
    def config_options(self):
        del self.options.potato
"""
        self.client.save({CONANFILE: conanfile}, clean_first=True)
        if export:
            self.client.run("export . lasote/stable")

    def test_reuse(self):
        self._export("Hello0")
        self._export("Hello1", ["Hello0/0.1@lasote/stable"])
        self._export("Hello2", ["Hello1/0.1@lasote/stable", "Hello0/0.1@lasote/stable"])
        self._export("Hello3", ["Hello2/0.1@lasote/stable"], export=False)

        self.client.run("install . --build missing")
        self.assertIn("conanfile.py (Hello3/0.1): Generated conaninfo.txt",
                      self.client.out)

    def test_check_duplicated_full_requires(self):
        self._export("Hello0")
        self._export("Hello1", ["Hello0/0.1@lasote/stable"])
        self._export("Hello2", ["Hello1/0.1@lasote/stable", "Hello0/0.1@lasote/stable"],
                     export=False)

        self.client.run("install . --build missing")
        self.assertIn("conanfile.py (Hello2/0.1): Generated conaninfo.txt",
                      self.client.out)
        conaninfo = self.client.load("conaninfo.txt")
        self.assertEqual(1, conaninfo.count("Hello0/0.1@lasote/stable"))



# coding=utf-8

import os
import textwrap
import unittest

from parameterized import parameterized

from conans.model.ref import ConanFileReference
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


class InfoCommandTest(unittest.TestCase):

    def setUp(self):
        self.ref = ConanFileReference.loads('lib/version@user/name')
        self.ref_child = ConanFileReference.loads('child/version@user/name')

        self.t = TestClient(path_with_spaces=False)
        self.t.save({'conanfile.py': GenConanfile()})
        self.t.run('create . parent/version@user/name')

        lib_folder = os.path.join(self.t.current_folder, 'lib')
        conan_package_layout = textwrap.dedent("""\
            [includedirs]
            src/include
            """)
        self.t.save({'lib/conanfile.py': GenConanfile().with_requires("parent/version@user/name"),
                     "lib/mylayout": conan_package_layout})
        self.t.run('editable add "{}" {}'.format(lib_folder, self.ref))
        self.assertTrue(self.t.cache.installed_as_editable(self.ref))

        # Create child
        self.t.save({'conanfile.py': GenConanfile().with_requires(self.ref)})
        self.t.run('export . {}'.format(self.ref_child))

    def tearDown(self):
        self.t.run('editable remove {}'.format(self.ref))
        self.assertFalse(self.t.cache.installed_as_editable(self.ref))

    @parameterized.expand([(True, ), (False, )])
    def test_no_args(self, use_local_path):
        project_name = "conanfile.py" if use_local_path else self.ref_child

        self.t.run('info {}'.format(project_name))
        revision = "    Revision: None\n"\
                   "    Package revision: None\n" \
                   if self.t.cache.config.revisions_enabled else ""
        self.assertIn("lib/version@user/name\n"
                      "    ID: e94ed0d45e4166d2f946107eaa208d550bf3691e\n"
                      "    BuildID: None\n"
                      "    Context: host\n"
                      "    Remote: None\n"
                      "    Provides: lib\n"
                      "    Recipe: Editable\n{}"
                      "    Binary: Editable\n"
                      "    Binary remote: None\n"
                      "    Required by:\n"
                      "        {}\n"
                      "    Requires:\n"
                      "        parent/version@user/name\n".format(revision, project_name),
                      self.t.out)

    @parameterized.expand([(True,), (False,)])
    def test_only_none(self, use_local_path):
        project_name = "conanfile.py" if use_local_path else self.ref_child

        self.t.run('info {} --only None'.format(project_name))
        # Compare, order is not guaranteed
        self.assertListEqual(sorted(str(self.t.out).splitlines()),
                             sorted(["lib/version@user/name",
                                     "parent/version@user/name",
                                     str(project_name)]))
